# -*- coding: utf-8 -*-
"""
Swift language-specific code review checklist.
"""

CHECKLIST = """
## Swift 代码审查检查清单

### 代码风格和格式
- [ ] 是否遵循 Swift API 设计指南
- [ ] 命名是否清晰明确 (驼峰式命名法)
- [ ] 是否使用 SwiftLint 等工具进行代码规范检查
- [ ] 缩进和格式是否一致
- [ ] 是否避免了过长的函数和过度嵌套

### Swift 语言特性
- [ ] 是否正确使用可选类型 (Optional)
- [ ] 是否安全地解包可选值
- [ ] 是否适当使用类型推断
- [ ] 是否合理使用值类型和引用类型
- [ ] 是否正确使用协议和扩展
- [ ] 是否适当使用属性观察器 (willSet/didSet)
- [ ] 是否正确使用闭包和逃逸闭包
- [ ] 是否合理使用 guard 语句和条件提前返回

### 内存管理
- [ ] 是否避免了循环引用 (使用 weak 或 unowned)
- [ ] 闭包捕获列表是否正确设置
- [ ] 是否正确处理委托和通知的生命周期
- [ ] 是否避免了内存泄漏

### 并发和异步
- [ ] 是否正确使用 GCD 或 Operation
- [ ] 是否避免了线程竞争
- [ ] UI 更新是否在主线程进行
- [ ] 是否正确处理异步操作的取消和超时
- [ ] Swift 5.5+ 是否合理使用结构化并发 (async/await)

### 性能优化
- [ ] 是否避免不必要的计算或对象创建
- [ ] 集合操作是否高效
- [ ] 是否避免了过度使用字符串操作
- [ ] 是否考虑了缓存策略
- [ ] 是否避免了过度使用计算属性

### 用户界面 (UI)
- [ ] 约束是否正确设置 (没有冲突或歧义)
- [ ] 是否考虑了不同屏幕尺寸和方向
- [ ] 是否遵循人机界面指南 (HIG)
- [ ] 是否适当使用 SF Symbols 和系统字体
- [ ] 是否适配了深色模式

### 安全性
- [ ] 敏感数据是否使用钥匙串存储
- [ ] 网络请求是否使用 HTTPS
- [ ] 输入验证是否充分
- [ ] 是否防范 JSON 解析错误
- [ ] 是否避免硬编码凭证

### 依赖管理
- [ ] 第三方依赖是否必要
- [ ] 依赖版本是否固定
- [ ] 是否考虑了依赖的维护状态和安全性
- [ ] 是否最小化了外部依赖

### 测试
- [ ] 是否有单元测试覆盖核心功能
- [ ] 是否使用 XCTest 框架进行测试
- [ ] 是否测试了错误路径和边界条件
- [ ] UI 测试是否覆盖主要用户流程
- [ ] 测试是否独立且可重复

### 文档和注释
- [ ] 是否使用 Xcode 的标记注释 (MARK, TODO, FIXME)
- [ ] 复杂逻辑是否有注释说明
- [ ] 公开 API 是否有文档注释
- [ ] 注释是否与代码保持同步
"""



# -*- coding: utf-8 -*-
"""
Python language-specific code review checklist.
"""

CHECKLIST = """
## Python 代码审查检查清单

### 代码风格
- [ ] 是否遵循 PEP 8 规范
- [ ] 命名是否符合 Python 约定
- [ ] 是否有适当的空格和缩进
- [ ] 是否避免了过长的行

### 类型安全
- [ ] 是否使用类型注解 (Python 3.6+)
- [ ] 是否处理了可能的类型不匹配
- [ ] 复杂参数是否有类型说明

### 异常处理
- [ ] 是否捕获和处理了异常
- [ ] 是否使用了合适的异常类型
- [ ] 是否避免了过于宽泛的 except 子句
- [ ] 自定义异常是否合理

### 性能
- [ ] 是否避免了低效的循环和操作
- [ ] 列表推导式和生成器的使用是否合理
- [ ] 是否优化了 I/O 操作
- [ ] 是否合理使用了缓存

### 安全
- [ ] 是否存在 SQL 注入风险
- [ ] 是否存在命令注入风险
- [ ] 密码和密钥是否安全存储
- [ ] 是否正确验证用户输入

### 依赖管理
- [ ] 是否明确指定了依赖版本
- [ ] 是否考虑了版本兼容性
- [ ] 是否最小化了依赖的数量

### 测试
- [ ] 是否有单元测试
- [ ] 测试覆盖率是否合理
- [ ] 是否处理了边界情况

### 文档
- [ ] 函数和类是否有文档字符串
- [ ] 复杂逻辑是否有注释说明
- [ ] README 是否完整

### 设计
- [ ] 是否遵循 SOLID 原则
- [ ] 是否避免了过度工程
- [ ] 是否实现了适当的抽象
"""



# -*- coding: utf-8 -*-
from typing import Any, Protocol, Tuple


class OutputHandlerProtocol(Protocol):
    """
    Defines the interface for an output handler, which is responsible for
    processing the model's response, typically to execute a tool.
    """

    def name(self) -> str:
        """Returns the name of the handler."""
        ...

    def can_handle(self, response: str) -> bool:
        """Determines if this handler can process the given response."""
        ...

    def prompt(self) -> str:
        """Returns the prompt snippet that describes the handler's functionality."""
        ...

    def handle(self, response: str, agent: Any) -> Tuple[bool, Any]:
        """
        Handles the response, executing the associated logic.

        Returns:
            A tuple containing a boolean (whether to return) and the result.
        """
        ...



# -*- coding: utf-8 -*-
from typing import Any, Tuple, TYPE_CHECKING

from jarvis.jarvis_utils.input import user_confirm
from jarvis.jarvis_utils.output import OutputType, PrettyOutput

if TYPE_CHECKING:
    from jarvis.jarvis_agent import Agent


def execute_tool_call(response: str, agent: "Agent") -> Tuple[bool, Any]:
    """
    Parses the model's response, identifies the appropriate tool, and executes it.

    Args:
        response: The response string from the model, potentially containing a tool call.
        agent: The agent instance, providing context like output handlers and settings.

    Returns:
        A tuple containing:
        - A boolean indicating if the tool's result should be returned to the user.
        - The result of the tool execution or an error message.
    """
    tool_list = []
    for handler in agent.output_handler:
        if handler.can_handle(response):
            tool_list.append(handler)

    if len(tool_list) > 1:
        error_message = (
            f"操作失败：检测到多个操作。一次只能执行一个操作。"
            f"尝试执行的操作：{', '.join([handler.name() for handler in tool_list])}"
        )
        PrettyOutput.print(error_message, OutputType.WARNING)
        return False, error_message

    if not tool_list:
        return False, ""

    tool_to_execute = tool_list[0]
    if not agent.execute_tool_confirm or user_confirm(
        f"需要执行{tool_to_execute.name()}确认执行？", True
    ):
        print(f"🔧 正在执行{tool_to_execute.name()}...")
        result = tool_to_execute.handle(response, agent)
        print(f"✅ {tool_to_execute.name()}执行完成")
        return result

    return False, ""



# -*- coding: utf-8 -*-
def ot(tag_name: str) -> str:
    """生成HTML标签开始标记

    参数：
    tag_name -- HTML标签名称

    返回：
    格式化的开始标签字符串
    """
    return f"<{tag_name}>"


def ct(tag_name: str) -> str:
    """生成HTML标签结束标记

    参数：
    tag_name -- HTML标签名称

    返回：
    格式化的结束标签字符串
    """
    return f"</{tag_name}>"



# -*- coding: utf-8 -*-
import os
import tempfile
from pathlib import Path
from typing import Any, Dict

from jarvis.jarvis_utils.output import OutputType, PrettyOutput


class ScriptTool:
    """Combined script execution tool

    Executes scripts with any interpreter with a unified interface.
    """

    name = "execute_script"
    description = (
        "执行脚本并返回结果，支持任意解释器。"
        + "注意：由于模型上下文长度限制，请避免在脚本中输出大量信息，应该使用rg过滤输出。"
        + "与virtual_tty不同，此工具会创建一个临时的脚本文件，并使用脚本命令执行脚本，不具备交互式操作的能力，"
        + "适用于需要执行脚本并获取结果的场景。不适合需要交互式操作的场景（如：ssh连接、sftp传输、gdb/dlv调试等）。"
    )
    parameters = {
        "type": "object",
        "properties": {
            "interpreter": {
                "type": "string",
                "description": "脚本解释器: 如bash, python3, expect, perl, ruby等任意解释器。如需直接执行shell命令, 可使用bash作为解释器",
            },
            "script_content": {"type": "string", "description": "要执行的脚本内容"},
        },
        "required": ["script_content"],
    }

    # Map of common file extensions for interpreters (can be extended as needed)
    INTERPRETER_EXTENSIONS = {
        "bash": "sh",
        "sh": "sh",
        "python": "py",
        "python2": "py",
        "python3": "py",
        "perl": "pl",
        "ruby": "rb",
        "node": "js",
        "nodejs": "js",
        "php": "php",
        "powershell": "ps1",
        "pwsh": "ps1",
        "R": "r",
        "Rscript": "r",
        "julia": "jl",
        "lua": "lua",
        "go": "go",
        "awk": "awk",
        "kotlin": "kt",
        "java": "java",
        "javac": "java",
        "scala": "scala",
        "swift": "swift",
        "gcc": "c",
        "g++": "cpp",
    }

    def get_display_output(self, file_path: str) -> str:
        """消除控制字符，得到用户实际看到的文本，去除script命令首尾行"""
        # 读取文件内容并尝试多种编码
        with open(file_path, "rb") as f:
            data = f.read()

        import pyte

        screen = pyte.Screen(300, 100000)
        stream = pyte.ByteStream(screen)
        stream.feed(data)

        # 清理每行右侧空格，并过滤空行
        cleaned = []
        cleaned = []
        for y in range(screen.lines):
            line = screen.buffer[y]
            stripped = "".join(char.data for char in line.values()).rstrip()
            if stripped:
                cleaned.append(stripped)
        return "\n".join(cleaned[1:-1])

    def _execute_script_with_interpreter(
        self, interpreter: str, script_content: str
    ) -> Dict[str, Any]:
        """Execute a script with the specified interpreter

        Args:
            interpreter: The interpreter to use (any valid interpreter command)
            script_content: Content of the script

        Returns:
            Dictionary with execution results
        """
        try:
            # Get file extension for the interpreter
            extension = self.INTERPRETER_EXTENSIONS.get(interpreter, "script")

            # Create temporary script file
            script_path = os.path.join(
                tempfile.gettempdir(),
                f"jarvis_{interpreter.replace('/', '_')}_{os.getpid()}.{extension}",
            )
            output_file = os.path.join(
                tempfile.gettempdir(), f"jarvis_output_{os.getpid()}.log"
            )
            try:
                with open(script_path, "w", encoding="utf-8", errors="ignore") as f:
                    f.write(script_content)

                # Use script command to capture both stdout and stderr
                tee_command = (
                    f"script -q -c '{interpreter} {script_path}' {output_file}"
                )

                # Execute command and capture return code
                os.system(tee_command)

                # Read and process output file
                try:
                    # 消除控制字符，得到用户实际看到的文本
                    output = self.get_display_output(output_file)
                except Exception as e:
                    output = f"读取输出文件失败: {str(e)}"

                # Return successful result
                return {
                    "success": True,
                    "stdout": output,
                    "stderr": "",
                }

            finally:
                # Clean up temporary files
                Path(script_path).unlink(missing_ok=True)
                Path(output_file).unlink(missing_ok=True)

        except Exception as e:
            PrettyOutput.print(str(e), OutputType.ERROR)
            return {"success": False, "stdout": "", "stderr": str(e)}

    def execute(self, args: Dict) -> Dict[str, Any]:
        """Execute script based on interpreter and content

        Args:
            args: Dictionary containing interpreter (or script_type) and script_content

        Returns:
            Dictionary with execution results
        """
        try:
            script_content = args.get("script_content", "").strip()
            if not script_content:
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": "Missing or empty script_content parameter",
                }

            # Get interpreter, default to bash if not specified
            interpreter = args.get("interpreter", "bash")

            # Execute the script with the specified interpreter
            return self._execute_script_with_interpreter(interpreter, script_content)

        except Exception as e:
            PrettyOutput.print(str(e), OutputType.ERROR)
            return {"success": False, "stdout": "", "stderr": str(e)}


if __name__ == "__main__":
    script_tool = ScriptTool()
    print(script_tool.get_display_output("/home/wangmaobin/code/Jarvis/a.txt"))



from abc import ABC, abstractmethod
import os
import os
from abc import ABC, abstractmethod

from jarvis.jarvis_agent import Agent as JarvisAgent
from jarvis.jarvis_platform.base import BasePlatform
from jarvis.jarvis_platform.registry import PlatformRegistry


class LLMInterface(ABC):
    """
    大型语言模型接口的抽象基类。

    该类定义了与远程LLM交互的标准接口。
    任何LLM提供商（如OpenAI、Anthropic等）都应作为该接口的子类来实现。
    """

    @abstractmethod
    def generate(self, prompt: str, **kwargs) -> str:
        """
        根据给定的提示从LLM生成响应。

        参数:
            prompt: 发送给LLM的输入提示。
            **kwargs: LLM API调用的其他关键字参数
                      （例如，temperature, max_tokens）。

        返回:
            由LLM生成的文本响应。
        """
        pass


class ToolAgent_LLM(LLMInterface):
    """
    LLMInterface的一个实现，它使用一个能操作工具的JarvisAgent来生成最终响应。
    """

    def __init__(self):
        """
        初始化工具-代理 LLM 包装器。
        """
        print("🤖 已初始化工具 Agent 作为最终应答者。")
        self.allowed_tools = ["read_code", "execute_script"]
        # 为代理提供一个通用的系统提示
        self.system_prompt = "You are a helpful assistant. Please answer the user's question based on the provided context. You can use tools to find more information if needed."
        self.summary_prompt = """
<report>
请为本次问答任务生成一个总结报告，包含以下内容：

1. **原始问题**: 重述用户最开始提出的问题。
2. **关键信息来源**: 总结你是基于哪些关键信息或文件得出的结论。
3. **最终答案**: 给出最终的、精炼的回答。
</report>
"""

    def generate(self, prompt: str, **kwargs) -> str:
        """
        使用受限的工具集运行JarvisAgent以生成答案。

        参数:
            prompt: 要发送给代理的完整提示，包括上下文。
            **kwargs: 已忽略，为保持接口兼容性而保留。

        返回:
            由代理生成的最终答案。
        """
        try:
            # 使用RAG上下文的特定设置初始化代理
            agent = JarvisAgent(
                system_prompt=self.system_prompt,
                use_tools=self.allowed_tools,
                auto_complete=True,
                use_methodology=False,
                use_analysis=False,
                need_summary=True,
                summary_prompt=self.summary_prompt,
            )

            # 代理的run方法需要'user_input'参数
            final_answer = agent.run(user_input=prompt)
            return str(final_answer)

        except Exception as e:
            print(f"❌ Agent 在执行过程中发生错误: {e}")
            return "错误: Agent 未能成功生成回答。"


class JarvisPlatform_LLM(LLMInterface):
    """
    项目内部平台的LLMInterface实现。

    该类使用PlatformRegistry来获取配置的“普通”模型。
    """

    def __init__(self):
        """
        初始化Jarvis平台LLM客户端。
        """
        try:
            self.registry = PlatformRegistry.get_global_platform_registry()
            self.platform: BasePlatform = self.registry.get_normal_platform()
            self.platform.set_suppress_output(False)  # 确保模型没有控制台输出
            print(f"🚀 已初始化 Jarvis 平台 LLM，模型: {self.platform.name()}")
        except Exception as e:
            print(f"❌ 初始化 Jarvis 平台 LLM 失败: {e}")
            raise

    def generate(self, prompt: str, **kwargs) -> str:
        """
        向本地平台模型发送提示并返回响应。

        参数:
            prompt: 用户的提示。
            **kwargs: 已忽略，为保持接口兼容性而保留。

        返回:
            由平台模型生成的响应。
        """
        try:
            # 使用健壮的chat_until_success方法
            return self.platform.chat_until_success(prompt)
        except Exception as e:
            print(f"❌ 调用 Jarvis 平台模型时发生错误: {e}")
            return "错误: 无法从本地LLM获取响应。"



# -*- coding: utf-8 -*-

# 人类交互平台实现模块

# 提供与真实人类交互的模拟接口

import json
import random
import string
from typing import Generator, List, Tuple

from jarvis.jarvis_platform.base import BasePlatform
from jarvis.jarvis_utils.input import get_multiline_input
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.utils import copy_to_clipboard


class HumanPlatform(BasePlatform):
    """人类交互平台实现，模拟大模型但实际上与人交互"""

    def get_model_list(self) -> List[Tuple[str, str]]:
        """获取支持的模型列表"""
        return [("human", "Human Interaction")]

    def __init__(self):
        """初始化人类交互平台"""
        super().__init__()
        self.conversation_id = ""  # 会话ID，用于标识当前对话
        self.model_name = "human"  # 默认模型名称
        self.system_message = ""  # 系统消息，用于初始化对话
        self.first_message = True

    def set_system_prompt(self, message: str):
        """设置系统消息"""
        self.system_message = message

    def set_model_name(self, model_name: str):
        """设置模型名称"""
        if model_name == "human":
            self.model_name = model_name
        else:
            PrettyOutput.print(f"错误：不支持的模型: {model_name}", OutputType.ERROR)

    def chat(self, message: str) -> Generator[str, None, None]:
        """发送消息并获取人类响应"""
        if not self.conversation_id:
            self.conversation_id = "".join(
                random.choices(string.ascii_letters + string.digits, k=8)
            )
            session_info = f"(会话ID: {self.conversation_id})"
        else:
            session_info = f"(会话ID: {self.conversation_id})"

        if self.system_message and self.first_message:
            prompt = f"{self.system_message}\n\n{message} {session_info}"
            self.first_message = False
        else:
            prompt = f"{message} {session_info}"

        # 将prompt复制到剪贴板
        copy_to_clipboard(prompt)

        response = get_multiline_input(prompt + "\n\n请回复:")
        yield response
        return None

    def upload_files(self, file_list: List[str]) -> bool:
        """文件上传功能，人类平台不需要实际处理"""
        PrettyOutput.print("人类交互平台不支持文件上传", OutputType.WARNING)
        return False

    def delete_chat(self) -> bool:
        """删除当前会话"""
        self.conversation_id = ""
        self.first_message = True
        return True

    def save(self, file_path: str) -> bool:
        """Save chat session to a file."""
        state = {
            "conversation_id": self.conversation_id,
            "model_name": self.model_name,
            "system_message": self.system_message,
            "first_message": self.first_message,
        }

        try:
            with open(file_path, "w", encoding="utf-8") as f:
                json.dump(state, f, ensure_ascii=False, indent=4)
            self._saved = True
            PrettyOutput.print(f"会话已成功保存到 {file_path}", OutputType.SUCCESS)
            return True
        except Exception as e:
            PrettyOutput.print(f"保存会话失败: {str(e)}", OutputType.ERROR)
            return False

    def restore(self, file_path: str) -> bool:
        """Restore chat session from a file."""
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                state = json.load(f)

            self.conversation_id = state.get("conversation_id", "")
            self.model_name = state.get("model_name", "human")
            self.system_message = state.get("system_message", "")
            self.first_message = state.get("first_message", True)
            self._saved = True

            PrettyOutput.print(f"从 {file_path} 成功恢复会话", OutputType.SUCCESS)
            return True
        except FileNotFoundError:
            PrettyOutput.print(f"会话文件未找到: {file_path}", OutputType.ERROR)
            return False
        except Exception as e:
            PrettyOutput.print(f"恢复会话失败: {str(e)}", OutputType.ERROR)
            return False

    def name(self) -> str:
        """平台名称"""
        return self.model_name

    @classmethod
    def platform_name(cls) -> str:
        """平台名称"""
        return "human"

    def support_web(self) -> bool:
        """是否支持网页浏览功能"""
        return False

    def support_upload_files(self) -> bool:
        """是否支持文件上传功能"""
        return False



# -*- coding: utf-8 -*-
from typing import List

from jarvis.jarvis_agent.protocols import OutputHandlerProtocol


def build_action_prompt(output_handlers: List[OutputHandlerProtocol]) -> str:
    """
    Builds the action prompt string from a list of output handlers.

    Args:
        output_handlers: A list of output handler instances.

    Returns:
        A formatted string containing the action prompt.
    """
    action_prompt = """
<actions>
# 🧰 可用操作
以下是您可以使用的操作：
"""

    # Add tool list overview
    action_prompt += "\n<overview>\n## Action List\n"
    action_prompt += (
        "[" + ", ".join([handler.name() for handler in output_handlers]) + "]"
    )
    action_prompt += "\n</overview>"

    # Add details for each tool
    action_prompt += "\n\n<details>\n# 📝 Action Details\n"
    for handler in output_handlers:
        action_prompt += f"\n<tool>\n## {handler.name()}\n"
        # Get the handler's prompt and ensure correct formatting
        handler_prompt = handler.prompt().strip()
        # Adjust indentation to maintain hierarchy
        handler_prompt = "\n".join(
            "   " + line if line.strip() else line
            for line in handler_prompt.split("\n")
        )
        action_prompt += handler_prompt + "\n</tool>\n"

    # Add tool usage summary
    action_prompt += """
</details>

<rules>
# ❗ 重要操作使用规则
1. 一次对话只能使用一个操作，否则会出错
2. 严格按照每个操作的格式执行
3. 等待操作结果后再进行下一个操作
4. 处理完结果后再调用新的操作
5. 如果对操作使用不清楚，请请求帮助
</rules>
</actions>
"""
    return action_prompt



# -*- coding: utf-8 -*-
import hashlib
import json
import os
import signal
import subprocess
import sys
import time
from pathlib import Path
from typing import Any, Callable, Dict, List, Optional
from datetime import datetime

import yaml  # type: ignore

from jarvis import __version__
from jarvis.jarvis_utils.config import (
    get_data_dir,
    get_max_big_content_size,
    set_global_env_data,
)
from jarvis.jarvis_utils.embedding import get_context_token_count
from jarvis.jarvis_utils.globals import get_in_chat, get_interrupt, set_interrupt
from jarvis.jarvis_utils.output import OutputType, PrettyOutput

g_config_file = None


def _setup_signal_handler() -> None:
    """设置SIGINT信号处理函数"""
    original_sigint = signal.getsignal(signal.SIGINT)

    def sigint_handler(signum, frame):
        if get_in_chat():
            set_interrupt(True)
            if get_interrupt() > 5 and original_sigint and callable(original_sigint):
                original_sigint(signum, frame)
        else:
            if original_sigint and callable(original_sigint):
                original_sigint(signum, frame)

    signal.signal(signal.SIGINT, sigint_handler)


def _show_welcome_message(welcome_str: str) -> None:
    """显示欢迎信息

    参数:
        welcome_str: 欢迎信息字符串
    """
    if not welcome_str:
        return

    jarvis_ascii_art = f"""
   ██╗ █████╗ ██████╗ ██╗   ██╗██╗███████╗
   ██║██╔══██╗██╔══██╗██║   ██║██║██╔════╝
   ██║███████║██████╔╝██║   ██║██║███████╗
██╗██║██╔══██║██╔══██╗╚██╗ ██╔╝██║╚════██║
╚████║██║  ██║██║  ██║ ╚████╔╝ ██║███████║
 ╚═══╝╚═╝  ╚═╝╚═╝  ╚═╝  ╚═══╝  ╚═╝╚══════╝
 {welcome_str}

 https://github.com/skyfireitdiy/Jarvis
 v{__version__}
"""
    PrettyOutput.print_gradient_text(jarvis_ascii_art, (0, 120, 255), (0, 255, 200))


def _check_git_updates() -> bool:
    """检查并更新git仓库

    返回:
        bool: 是否需要重启进程
    """
    script_dir = Path(os.path.dirname(os.path.dirname(__file__)))
    from jarvis.jarvis_utils.git_utils import check_and_update_git_repo

    return check_and_update_git_repo(str(script_dir))


def init_env(welcome_str: str, config_file: Optional[str] = None) -> None:
    """初始化Jarvis环境

    参数:
        welcome_str: 欢迎信息字符串
        config_file: 配置文件路径，默认为None(使用~/.jarvis/config.yaml)
    """
    # 1. 设置信号处理
    _setup_signal_handler()

    # 2. 统计命令使用
    count_cmd_usage()

    # 3. 显示欢迎信息
    if welcome_str:
        _show_welcome_message(welcome_str)

    # 4. 设置配置文件
    global g_config_file
    g_config_file = config_file
    load_config()

    # 5. 检查git更新
    if _check_git_updates():
        os.execv(sys.executable, [sys.executable] + sys.argv)
        sys.exit(0)


def load_config():
    config_file = g_config_file
    config_file_path = (
        Path(config_file)
        if config_file is not None
        else Path(os.path.expanduser("~/.jarvis/config.yaml"))
    )

    # 加载配置文件
    if not config_file_path.exists():
        old_config_file = config_file_path.parent / "env"
        if old_config_file.exists():  # 旧的配置文件存在
            _read_old_config_file(old_config_file)
        else:
            # 生成默认配置文件
            schema_path = (
                Path(__file__).parent.parent / "jarvis_data" / "config_schema.json"
            )
            if schema_path.exists():
                try:
                    config_file_path.parent.mkdir(parents=True, exist_ok=True)
                    generate_default_config(str(schema_path), str(config_file_path))
                    PrettyOutput.print(
                        f"已生成默认配置文件: {config_file_path}", OutputType.INFO
                    )
                    sys.exit(0)
                except Exception as e:
                    PrettyOutput.print(f"生成默认配置文件失败: {e}", OutputType.ERROR)
    else:
        _load_and_process_config(str(config_file_path.parent), str(config_file_path))


from typing import Tuple


def _load_config_file(config_file: str) -> Tuple[str, dict]:
    """读取并解析YAML格式的配置文件

    参数:
        config_file: 配置文件路径

    返回:
        Tuple[str, dict]: (文件原始内容, 解析后的配置字典)
    """
    with open(config_file, "r", encoding="utf-8") as f:
        content = f.read()
        config_data = yaml.safe_load(content) or {}
        return content, config_data


def _ensure_schema_declaration(
    jarvis_dir: str, config_file: str, content: str, config_data: dict
) -> None:
    """确保配置文件包含schema声明

    参数:
        jarvis_dir: Jarvis数据目录路径
        config_file: 配置文件路径
        content: 配置文件原始内容
        config_data: 解析后的配置字典
    """
    if (
        isinstance(config_data, dict)
        and "# yaml-language-server: $schema=" not in content
    ):
        schema_path = Path(
            os.path.relpath(
                Path(__file__).parent.parent / "jarvis_data" / "config_schema.json",
                start=jarvis_dir,
            )
        )
        with open(config_file, "w", encoding="utf-8") as f:
            f.write(f"# yaml-language-server: $schema={schema_path}\n")
            f.write(content)


def _process_env_variables(config_data: dict) -> None:
    """处理配置中的环境变量

    参数:
        config_data: 解析后的配置字典
    """
    if "ENV" in config_data and isinstance(config_data["ENV"], dict):
        os.environ.update(
            {str(k): str(v) for k, v in config_data["ENV"].items() if v is not None}
        )


def _load_and_process_config(jarvis_dir: str, config_file: str) -> None:
    """加载并处理配置文件

    功能：
    1. 读取配置文件
    2. 确保schema声明存在
    3. 保存配置到全局变量
    4. 处理环境变量

    参数:
        jarvis_dir: Jarvis数据目录路径
        config_file: 配置文件路径
    """
    content, config_data = _load_config_file(config_file)
    _ensure_schema_declaration(jarvis_dir, config_file, content, config_data)
    set_global_env_data(config_data)
    _process_env_variables(config_data)


def generate_default_config(schema_path: str, output_path: str) -> None:
    """从schema文件生成默认的YAML格式配置文件

    功能：
    1. 从schema文件读取配置结构
    2. 根据schema中的default值生成默认配置
    3. 自动添加schema声明
    4. 处理嵌套的schema结构
    5. 保留注释和格式

    参数:
        schema_path: schema文件路径
        output_path: 生成的配置文件路径
    """
    with open(schema_path, "r", encoding="utf-8") as f:
        schema = json.load(f)

    def _generate_from_schema(schema_dict: Dict[str, Any]) -> Dict[str, Any]:
        config = {}
        if "properties" in schema_dict:
            for key, value in schema_dict["properties"].items():
                if "default" in value:
                    config[key] = value["default"]
                elif "properties" in value:  # 处理嵌套对象
                    config[key] = _generate_from_schema(value)
                elif value.get("type") == "array":  # 处理列表类型
                    config[key] = []
        return config

    default_config = _generate_from_schema(schema)

    # 添加schema声明
    rel_schema_path = Path(
        os.path.relpath(
            Path(schema_path),
            start=Path(output_path).parent,
        )
    )
    content = f"# yaml-language-server: $schema={rel_schema_path}\n"
    content += yaml.dump(default_config, allow_unicode=True, sort_keys=False)

    with open(output_path, "w", encoding="utf-8") as f:
        f.write(content)


def _read_old_config_file(config_file):
    """读取并解析旧格式的env配置文件

    功能：
    1. 解析键值对格式的旧配置文件
    2. 支持多行值的处理
    3. 自动去除值的引号和空格
    4. 将配置数据保存到全局变量
    5. 设置环境变量并显示迁移警告

    参数:
        config_file: 旧格式配置文件路径
    """
    config_data = {}
    current_key = None
    current_value = []
    with open(config_file, "r", encoding="utf-8", errors="ignore") as f:
        for line in f:
            line = line.rstrip()
            if not line or line.startswith(("#", ";")):
                continue
            if "=" in line and not line.startswith((" ", "\t")):
                # 处理之前收集的多行值
                if current_key is not None:
                    value = "\n".join(current_value).strip().strip("'").strip('"')
                    # 将字符串"true"/"false"转换为bool类型
                    if value.lower() == "true":
                        value = True
                    elif value.lower() == "false":
                        value = False
                    config_data[current_key] = value
                    current_value = []
                    # 解析新的键值对
                key, value = line.split("=", 1)
                current_key = key.strip()
                current_value.append(value.strip())
            elif current_key is not None:
                # 多行值的后续行
                current_value.append(line.strip())
                # 处理最后一个键值对
        if current_key is not None:
            value = "\n".join(current_value).strip().strip("'").strip('"')
            # 将字符串"true"/"false"转换为bool类型
            if value.lower() == "true":
                value = True
            elif value.lower() == "false":
                value = False
            config_data[current_key] = value
        os.environ.update(
            {str(k): str(v) for k, v in config_data.items() if v is not None}
        )
        set_global_env_data(config_data)
    PrettyOutput.print(
        f"检测到旧格式配置文件，旧格式以后将不再支持，请尽快迁移到新格式",
        OutputType.WARNING,
    )


def while_success(func: Callable[[], Any], sleep_time: float = 0.1) -> Any:
    """循环执行函数直到成功

    参数：
    func -- 要执行的函数
    sleep_time -- 每次失败后的等待时间（秒）

    返回：
    函数执行结果
    """
    while True:
        try:
            return func()
        except Exception as e:
            PrettyOutput.print(
                f"执行失败: {str(e)}, 等待 {sleep_time}s...", OutputType.WARNING
            )
            time.sleep(sleep_time)
            continue


def while_true(func: Callable[[], bool], sleep_time: float = 0.1) -> Any:
    """循环执行函数直到返回True

    参数:
        func: 要执行的函数，必须返回布尔值
        sleep_time: 每次失败后的等待时间(秒)

    返回:
        函数最终返回的True值

    注意:
        与while_success不同，此函数只检查返回是否为True，
        不捕获异常，异常会直接抛出
    """
    while True:
        ret = func()
        if ret:
            break
        PrettyOutput.print(f"执行失败, 等待 {sleep_time}s...", OutputType.WARNING)
        time.sleep(sleep_time)
    return ret


def get_file_md5(filepath: str) -> str:
    """计算文件内容的MD5哈希值

    参数:
        filepath: 要计算哈希的文件路径

    返回:
        str: 文件内容的MD5哈希值
    """
    return hashlib.md5(open(filepath, "rb").read(100 * 1024 * 1024)).hexdigest()


def get_file_line_count(filename: str) -> int:
    """计算文件中的行数

    参数:
        filename: 要计算行数的文件路径

    返回:
        int: 文件中的行数，如果文件无法读取则返回0
    """
    try:
        return len(open(filename, "r", encoding="utf-8", errors="ignore").readlines())
    except Exception as e:
        return 0


def _get_cmd_stats() -> Dict[str, int]:
    """从数据目录获取命令调用统计"""
    stats_file = Path(get_data_dir()) / "cmd_stat.yaml"
    if stats_file.exists():
        try:
            with open(stats_file, "r", encoding="utf-8") as f:
                return yaml.safe_load(f) or {}
        except Exception as e:
            PrettyOutput.print(f"加载命令调用统计失败: {str(e)}", OutputType.WARNING)
    return {}


def _update_cmd_stats(cmd_name: str) -> None:
    """更新命令调用统计"""
    stats = _get_cmd_stats()
    stats[cmd_name] = stats.get(cmd_name, 0) + 1
    stats_file = Path(get_data_dir()) / "cmd_stat.yaml"
    try:
        with open(stats_file, "w", encoding="utf-8") as f:
            yaml.safe_dump(stats, f, allow_unicode=True)
    except Exception as e:
        PrettyOutput.print(f"保存命令调用统计失败: {str(e)}", OutputType.WARNING)


def count_cmd_usage() -> None:
    """统计当前命令的使用次数"""
    import sys

    _update_cmd_stats(sys.argv[0])


def is_context_overflow(
    content: str, model_group_override: Optional[str] = None
) -> bool:
    """判断文件内容是否超出上下文限制"""
    return get_context_token_count(content) > get_max_big_content_size(
        model_group_override
    )


def get_loc_stats() -> str:
    """使用loc命令获取当前目录的代码统计信息

    返回:
        str: loc命令输出的原始字符串，失败时返回空字符串
    """
    try:
        result = subprocess.run(["loc"], capture_output=True, text=True)
        return result.stdout if result.returncode == 0 else ""
    except FileNotFoundError:
        return ""


def copy_to_clipboard(text: str) -> None:
    """将文本复制到剪贴板，依次尝试xsel和xclip (非阻塞)

    参数:
        text: 要复制的文本
    """
    print("--- 剪贴板内容开始 ---")
    print(text)
    print("--- 剪贴板内容结束 ---")
    # 尝试使用 xsel
    try:
        process = subprocess.Popen(
            ["xsel", "-b", "-i"],
            stdin=subprocess.PIPE,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )
        if process.stdin:
            process.stdin.write(text.encode("utf-8"))
            process.stdin.close()
        return
    except FileNotFoundError:
        pass  # xsel 未安装，继续尝试下一个
    except Exception as e:
        PrettyOutput.print(f"使用xsel时出错: {e}", OutputType.WARNING)

    # 尝试使用 xclip
    try:
        process = subprocess.Popen(
            ["xclip", "-selection", "clipboard"],
            stdin=subprocess.PIPE,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )
        if process.stdin:
            process.stdin.write(text.encode("utf-8"))
            process.stdin.close()
        return
    except FileNotFoundError:
        PrettyOutput.print(
            "xsel 和 xclip 均未安装, 无法复制到剪贴板", OutputType.WARNING
        )
    except Exception as e:
        PrettyOutput.print(f"使用xclip时出错: {e}", OutputType.WARNING)


def _pull_git_repo(repo_path: Path, repo_type: str):
    """对指定的git仓库执行git pull操作，并根据commit hash判断是否有更新。"""
    git_dir = repo_path / ".git"
    if not git_dir.is_dir():
        return

    PrettyOutput.print(f"正在更新{repo_type}库 '{repo_path.name}'...", OutputType.INFO)
    try:
        # 检查是否有远程仓库
        remote_result = subprocess.run(
            ["git", "remote"],
            cwd=repo_path,
            capture_output=True,
            text=True,
            check=True,
            timeout=10,
        )
        if not remote_result.stdout.strip():
            PrettyOutput.print(
                f"'{repo_path.name}' 未配置远程仓库，跳过更新。",
                OutputType.INFO,
            )
            return

        # 检查git仓库状态
        status_result = subprocess.run(
            ["git", "status", "--porcelain"],
            cwd=repo_path,
            capture_output=True,
            text=True,
            check=True,
            timeout=10,
        )
        if status_result.stdout:
            PrettyOutput.print(
                f"检测到 '{repo_path.name}' 存在未提交的更改，跳过自动更新。",
                OutputType.WARNING,
            )
            return

        # 获取更新前的commit hash
        before_hash_result = subprocess.run(
            ["git", "rev-parse", "HEAD"],
            cwd=repo_path,
            capture_output=True,
            text=True,
            check=True,
            timeout=10,
        )
        before_hash = before_hash_result.stdout.strip()

        # 执行 git pull
        pull_result = subprocess.run(
            ["git", "pull"],
            cwd=repo_path,
            capture_output=True,
            text=True,
            check=True,
            timeout=60,
        )

        # 获取更新后的commit hash
        after_hash_result = subprocess.run(
            ["git", "rev-parse", "HEAD"],
            cwd=repo_path,
            capture_output=True,
            text=True,
            check=True,
            timeout=10,
        )
        after_hash = after_hash_result.stdout.strip()

        if before_hash != after_hash:
            PrettyOutput.print(f"{repo_type}库 '{repo_path.name}' 已更新。", OutputType.SUCCESS)
            if pull_result.stdout.strip():
                PrettyOutput.print(pull_result.stdout.strip(), OutputType.INFO)
        else:
            PrettyOutput.print(f"{repo_type}库 '{repo_path.name}' 已是最新版本。", OutputType.INFO)

    except FileNotFoundError:
        PrettyOutput.print(
            f"git 命令未找到，跳过更新 '{repo_path.name}'。", OutputType.WARNING
        )
    except subprocess.TimeoutExpired:
        PrettyOutput.print(f"更新 '{repo_path.name}' 超时。", OutputType.ERROR)
    except subprocess.CalledProcessError as e:
        error_message = e.stderr.strip() if e.stderr else str(e)
        PrettyOutput.print(
            f"更新 '{repo_path.name}' 失败: {error_message}", OutputType.ERROR
        )
    except Exception as e:
        PrettyOutput.print(
            f"更新 '{repo_path.name}' 时发生未知错误: {str(e)}", OutputType.ERROR
        )


def daily_check_git_updates(repo_dirs: List[str], repo_type: str):
    """
    对指定的目录列表执行每日一次的git更新检查。

    Args:
        repo_dirs (List[str]): 需要检查的git仓库目录列表。
        repo_type (str): 仓库的类型名称，例如 "工具" 或 "方法论"，用于日志输出。
    """
    data_dir = Path(get_data_dir())
    last_check_file = data_dir / f"{repo_type}_updates_last_check.txt"
    should_check_for_updates = True

    if last_check_file.exists():
        try:
            last_check_timestamp = float(last_check_file.read_text())
            last_check_date = datetime.fromtimestamp(last_check_timestamp).date()
            if last_check_date == datetime.now().date():
                should_check_for_updates = False
        except (ValueError, IOError):
            pass

    if should_check_for_updates:
        PrettyOutput.print(f"执行每日{repo_type}库更新检查...", OutputType.INFO)
        for repo_dir in repo_dirs:
            p_repo_dir = Path(repo_dir)
            if p_repo_dir.exists() and p_repo_dir.is_dir():
                _pull_git_repo(p_repo_dir, repo_type)
        try:
            last_check_file.write_text(str(time.time()))
        except IOError as e:
            PrettyOutput.print(f"无法写入git更新检查时间戳: {e}", OutputType.WARNING)



# -*- coding: utf-8 -*-
# 标准库导入
import datetime
import os
import platform
from typing import Any, Callable, Dict, List, Optional, Protocol, Tuple, Union

# 第三方库导入

# 本地库导入
# jarvis_agent 相关
from jarvis.jarvis_agent.prompt_builder import build_action_prompt
from jarvis.jarvis_agent.protocols import OutputHandlerProtocol
from jarvis.jarvis_agent.session_manager import SessionManager
from jarvis.jarvis_agent.tool_executor import execute_tool_call
from jarvis.jarvis_agent.prompts import (
    DEFAULT_SUMMARY_PROMPT,
    SUMMARY_REQUEST_PROMPT,
    TASK_ANALYSIS_PROMPT,
)

# jarvis_platform 相关
from jarvis.jarvis_platform.base import BasePlatform
from jarvis.jarvis_platform.registry import PlatformRegistry

# jarvis_utils 相关
from jarvis.jarvis_utils.config import (
    get_max_token_count,
    get_normal_model_name,
    get_normal_platform_name,
    get_thinking_model_name,
    get_thinking_platform_name,
    is_execute_tool_confirm,
    is_use_analysis,
    is_use_methodology,
)
from jarvis.jarvis_utils.embedding import get_context_token_count
from jarvis.jarvis_utils.globals import (
    delete_agent,
    get_interrupt,
    make_agent_name,
    set_agent,
    set_interrupt,
)
from jarvis.jarvis_utils.input import get_multiline_input, user_confirm
from jarvis.jarvis_utils.methodology import load_methodology, upload_methodology
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.tag import ct, ot

origin_agent_system_prompt = f"""
<role>
# 🤖 角色
你是一个专业的任务执行助手，根据用户需求制定并执行详细的计划。
</role>

<rules>
# ❗ 核心规则
1.  **单步操作**: 每个响应必须包含且仅包含一个工具调用。
2.  **任务终结**: 当任务完成时，明确指出任务已完成。这是唯一可以不调用工具的例外。
3.  **无响应错误**: 空响应或仅有分析无工具调用的响应是致命错误，会导致系统挂起。
4.  **决策即工具**: 所有的决策和分析都必须通过工具调用来体现。
5.  **等待结果**: 在继续下一步之前，必须等待当前工具的执行结果。
6.  **持续推进**: 除非任务完成，否则必须生成可操作的下一步。
7.  **记录沉淀**: 如果解决方案有普适价值，应记录为方法论。
8.  **用户语言**: 始终使用用户的语言进行交流。
</rules>

<workflow>
# 🔄 工作流程
1.  **分析**: 理解和分析问题，定义清晰的目标。
2.  **设计**: 设计解决方案并制定详细的行动计划。
3.  **执行**: 按照计划，一次一个步骤地执行。
4.  **完成**: 验证任务是否达成目标，并进行总结。
</workflow>

<system_info>
# 系统信息
- OS: {platform.platform()} {platform.version()}
- Time: {datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")}
</system_info>
"""


class Agent:
    def clear(self):
        """
        Clears the current conversation history by delegating to the session manager.
        """
        self.session.clear()

    def __del__(self):
        # 只有在记录启动时才停止记录
        delete_agent(self.name)

    def get_tool_usage_prompt(self) -> str:
        """获取工具使用提示"""
        return build_action_prompt(self.output_handler)  # type: ignore

    def __init__(
        self,
        system_prompt: str,
        name: str = "Jarvis",
        description: str = "",
        llm_type: str = "normal",
        model_group: Optional[str] = None,
        summary_prompt: Optional[str] = None,
        auto_complete: bool = False,
        output_handler: List[OutputHandlerProtocol] = [],
        use_tools: List[str] = [],
        input_handler: Optional[List[Callable[[str, Any], Tuple[str, bool]]]] = None,
        execute_tool_confirm: Optional[bool] = None,
        need_summary: bool = True,
        multiline_inputer: Optional[Callable[[str], str]] = None,
        use_methodology: Optional[bool] = None,
        use_analysis: Optional[bool] = None,
        files: List[str] = [],
    ):
        self.files = files
        """初始化Jarvis Agent实例

        参数:
            system_prompt: 系统提示词，定义Agent的行为准则
            name: Agent名称，默认为"Jarvis"
            description: Agent描述信息
            llm_type: LLM类型，可以是 'normal' 或 'thinking'
            summary_prompt: 任务总结提示模板
            auto_complete: 是否自动完成任务
            output_handler: 输出处理器列表
            input_handler: 输入处理器列表
            max_context_length: 最大上下文长度
            execute_tool_confirm: 执行工具前是否需要确认
            need_summary: 是否需要生成总结
            multiline_inputer: 多行输入处理器
            use_methodology: 是否使用方法论
            use_analysis: 是否使用任务分析
        """
        self.name = make_agent_name(name)
        self.description = description
        # 初始化平台和模型
        if llm_type == "thinking":
            platform_name = get_thinking_platform_name(model_group)
            model_name = get_thinking_model_name(model_group)
        else:  # 默认为 normal
            platform_name = get_normal_platform_name(model_group)
            model_name = get_normal_model_name(model_group)

        self.model = PlatformRegistry().create_platform(platform_name)
        if self.model is None:
            PrettyOutput.print(
                f"平台 {platform_name} 不存在，将使用普通模型", OutputType.WARNING
            )
            self.model = PlatformRegistry().get_normal_platform()

        if model_name:
            self.model.set_model_name(model_name)

        self.model.set_model_group(model_group)

        self.user_data: Dict[str, Any] = {}

        self.model.set_suppress_output(False)

        # Initialize the session manager
        self.session = SessionManager(model=self.model, agent_name=self.name)

        from jarvis.jarvis_tools.registry import ToolRegistry

        self.output_handler = output_handler if output_handler else [ToolRegistry()]
        self.set_use_tools(use_tools)

        self.multiline_inputer = (
            multiline_inputer if multiline_inputer else get_multiline_input
        )

        # 如果有上传文件，自动禁用方法论
        self.use_methodology = (
            False
            if files
            else (
                use_methodology if use_methodology is not None else is_use_methodology()
            )
        )
        self.use_analysis = (
            use_analysis if use_analysis is not None else is_use_analysis()
        )
        self.system_prompt = system_prompt
        self.input_handler = input_handler if input_handler is not None else []
        self.need_summary = need_summary
        # Load configuration from environment variables

        self.after_tool_call_cb: Optional[Callable[[Agent], None]] = None

        self.execute_tool_confirm = (
            execute_tool_confirm
            if execute_tool_confirm is not None
            else is_execute_tool_confirm()
        )

        self.summary_prompt = (
            summary_prompt if summary_prompt else DEFAULT_SUMMARY_PROMPT
        )

        self.max_token_count = get_max_token_count(model_group)
        self.auto_complete = auto_complete
        welcome_message = f"{name} 初始化完成 - 使用 {self.model.name()} 模型"

        PrettyOutput.print(welcome_message, OutputType.SYSTEM)

        action_prompt = self.get_tool_usage_prompt()

        self.model.set_system_prompt(
            f"""
{self.system_prompt}

{action_prompt}
"""
        )
        self.first = True
        self.run_input_handlers_next_turn = False

    def set_user_data(self, key: str, value: Any):
        """Sets user data in the session."""
        self.session.set_user_data(key, value)

    def get_user_data(self, key: str) -> Optional[Any]:
        """Gets user data from the session."""
        return self.session.get_user_data(key)

    def set_use_tools(self, use_tools):
        """设置要使用的工具列表"""
        from jarvis.jarvis_tools.registry import ToolRegistry

        for handler in self.output_handler:
            if isinstance(handler, ToolRegistry):
                if use_tools:
                    handler.use_tools(use_tools)
                break

    def set_addon_prompt(self, addon_prompt: str):
        """Sets the addon prompt in the session."""
        self.session.set_addon_prompt(addon_prompt)

    def set_run_input_handlers_next_turn(self, value: bool):
        """Sets the flag to run input handlers on the next turn."""
        self.run_input_handlers_next_turn = value

    def set_after_tool_call_cb(self, cb: Callable[[Any], None]):  # type: ignore
        """设置工具调用后回调函数。

        参数:
            cb: 回调函数
        """
        self.after_tool_call_cb = cb

    def save_session(self) -> bool:
        """Saves the current session state by delegating to the session manager."""
        return self.session.save_session()

    def restore_session(self) -> bool:
        """Restores the session state by delegating to the session manager."""
        if self.session.restore_session():
            self.first = False
            return True
        return False

    def get_tool_registry(self) -> Optional[Any]:
        """获取工具注册表实例"""
        from jarvis.jarvis_tools.registry import ToolRegistry

        for handler in self.output_handler:
            if isinstance(handler, ToolRegistry):
                return handler
        return None

    def _call_model(
        self, message: str, need_complete: bool = False, run_input_handlers: bool = True
    ) -> str:
        """调用AI模型并实现重试逻辑

        参数:
            message: 输入给模型的消息
            need_complete: 是否需要完成任务标记

        返回:
            str: 模型的响应

        注意:
            1. 将使用指数退避重试，最多重试30秒
            2. 会自动处理输入处理器链
            3. 会自动添加附加提示
            4. 会检查并处理上下文长度限制
        """
        if run_input_handlers:
            for handler in self.input_handler:
                message, need_return = handler(message, self)
                if need_return:
                    return message

        if self.session.addon_prompt:
            message += f"\n\n{self.session.addon_prompt}"
            self.session.addon_prompt = ""
        else:
            message += f"\n\n{self.make_default_addon_prompt(need_complete)}"

        # 累加对话长度
        self.session.conversation_length += get_context_token_count(message)

        if self.session.conversation_length > self.max_token_count:
            message = self._summarize_and_clear_history() + "\n\n" + message
            self.session.conversation_length += get_context_token_count(message)

        if not self.model:
            raise RuntimeError("Model not initialized")
        response = self.model.chat_until_success(message)  # type: ignore
        self.session.conversation_length += get_context_token_count(response)

        return response

    def generate_summary(self) -> str:
        """生成对话历史摘要

        返回:
            str: 包含对话摘要的字符串

        注意:
            仅生成摘要，不修改对话状态
        """
        print("📄 正在总结对话历史...")
        try:
            if not self.model:
                raise RuntimeError("Model not initialized")
            summary = self.model.chat_until_success(
                self.session.prompt + "\n" + SUMMARY_REQUEST_PROMPT
            )  # type: ignore
            print("✅ 总结对话历史完成")
            return summary
        except Exception as e:
            print("❌ 总结对话历史失败")
            return ""

    def _summarize_and_clear_history(self) -> str:
        """总结当前对话并清理历史记录

        该方法将:
        1. 调用_generate_summary生成摘要
        2. 清除对话历史
        3. 保留系统消息
        4. 添加摘要作为新上下文
        5. 重置对话长度计数器

        返回:
            str: 包含对话摘要的字符串

        注意:
            当上下文长度超过最大值时使用
        """
        need_summary = True
        tmp_file_name = ""
        try:
            if self.model and self.model.support_upload_files():
                need_summary = False
            if need_summary:
                summary = self.generate_summary()
            else:
                import tempfile

                tmp_file = tempfile.NamedTemporaryFile(delete=False)
                tmp_file_name = tmp_file.name
            self.clear_history()  # type: ignore

            if need_summary:
                if not summary:
                    return ""

                return f"""
以下是之前对话的关键信息总结：

<content>
{summary}
</content>

请基于以上信息继续完成任务。请注意，这是之前对话的摘要，上下文长度已超过限制而被重置。请直接继续任务，无需重复已完成的步骤。如有需要，可以询问用户以获取更多信息。
        """
            else:
                if self.model and self.model.upload_files([tmp_file_name]):
                    return "上传的文件是历史对话信息，请基于历史对话信息继续完成任务。"
                else:
                    return ""
        finally:
            if tmp_file_name:
                os.remove(tmp_file_name)

    def _call_tools(self, response: str) -> Tuple[bool, Any]:
        """
        Delegates the tool execution to the external `execute_tool_call` function.
        """
        return execute_tool_call(response, self)

    def _complete_task(self) -> str:
        """完成任务并生成总结(如果需要)

        返回:
            str: 任务总结或完成状态

        注意:
            1. 对于主Agent: 可能会生成方法论(如果启用)
            2. 对于子Agent: 可能会生成总结(如果启用)
            3. 使用spinner显示生成状态
        """
        if self.use_analysis:
            self._analysis_task()
        if self.need_summary:
            print("📄 正在生成总结...")
            self.session.prompt = self.summary_prompt
            if not self.model:
                raise RuntimeError("Model not initialized")
            ret = self.model.chat_until_success(self.session.prompt)  # type: ignore
            print("✅ 总结生成完成")
            return ret

        return "任务完成"

    def _analysis_task(self):
        print("🔍 正在分析任务...")
        try:
            # 让模型判断是否需要生成方法论
            analysis_prompt = TASK_ANALYSIS_PROMPT

            self.session.prompt = analysis_prompt
            if not self.model:
                raise RuntimeError("Model not initialized")
            response = self.model.chat_until_success(self.session.prompt)  # type: ignore
            self._call_tools(response)
            print("✅ 分析完成")
        except Exception as e:
            print("❌ 分析失败")

    def make_default_addon_prompt(self, need_complete: bool) -> str:
        """生成附加提示。

        参数:
            need_complete: 是否需要完成任务

        """
        # 结构化系统指令
        action_handlers = ", ".join([handler.name() for handler in self.output_handler])

        # 任务完成提示
        complete_prompt = (
            f"- 输出{ot('!!!COMPLETE!!!')}"
            if need_complete and self.auto_complete
            else ""
        )

        addon_prompt = f"""
<system_prompt>
    请判断是否已经完成任务，如果已经完成：
    - 直接输出完成原因，不需要再有新的操作，不要输出{ot("TOOL_CALL")}标签
    {complete_prompt}
    如果没有完成，请进行下一步操作：
    - 仅包含一个操作
    - 如果信息不明确，请请求用户补充
    - 如果执行过程中连续失败5次，请使用ask_user询问用户操作
    - 操作列表：{action_handlers}
</system_prompt>

请继续。
"""

        return addon_prompt

    def run(self, user_input: str) -> Any:
        """处理用户输入并执行任务

        参数:
            user_input: 任务描述或请求

        返回:
            str|Dict: 任务总结报告或要发送的消息

        注意:
            1. 这是Agent的主运行循环
            2. 处理完整的任务生命周期
            3. 包含错误处理和恢复逻辑
            4. 自动加载相关方法论(如果是首次运行)
        """

        self.session.prompt = f"{user_input}"
        try:
            set_agent(self.name, self)

            run_input_handlers = True
            while True:
                if self.run_input_handlers_next_turn:
                    run_input_handlers = True
                    self.run_input_handlers_next_turn = False

                if self.first:
                    self._first_run()
                try:
                    current_response = self._call_model(
                        self.session.prompt, True, run_input_handlers
                    )
                    self.session.prompt = ""
                    run_input_handlers = False

                    if get_interrupt():
                        set_interrupt(False)
                        user_input = self.multiline_inputer(
                            f"模型交互期间被中断，请输入用户干预信息："
                        )
                        if user_input:
                            run_input_handlers = True
                            # 如果有工具调用且用户确认继续，则将干预信息和工具执行结果拼接为prompt
                            if any(
                                handler.can_handle(current_response)
                                for handler in self.output_handler
                            ):
                                if user_confirm(
                                    "检测到有工具调用，是否继续处理工具调用？", True
                                ):
                                    self.session.prompt = (
                                        f"{user_input}\n\n{current_response}"
                                    )
                                    run_input_handlers = False
                                    continue
                            self.session.prompt += f"{user_input}"
                            continue

                    need_return, self.session.prompt = self._call_tools(
                        current_response
                    )

                    if need_return:
                        return self.session.prompt

                    if self.after_tool_call_cb:
                        self.after_tool_call_cb(self)

                    if self.session.prompt or self.session.addon_prompt:
                        continue

                    if self.auto_complete and ot("!!!COMPLETE!!!") in current_response:
                        return self._complete_task()

                    # 获取用户输入
                    user_input = self.multiline_inputer(
                        f"{self.name}: 请输入，或输入空行来结束当前任务："
                    )

                    if user_input:
                        self.session.prompt = user_input
                        run_input_handlers = True
                        continue

                    if not user_input:
                        return self._complete_task()

                except Exception as e:
                    PrettyOutput.print(f"任务失败: {str(e)}", OutputType.ERROR)
                    return f"Task failed: {str(e)}"

        except Exception as e:
            PrettyOutput.print(f"任务失败: {str(e)}", OutputType.ERROR)
            return f"Task failed: {str(e)}"

    def _first_run(self):
        # 如果有上传文件，先上传文件
        if self.model and self.model.support_upload_files():
            if self.use_methodology:
                if not upload_methodology(self.model, other_files=self.files):
                    if self.files:
                        PrettyOutput.print(
                            "文件上传失败，将忽略文件列表", OutputType.WARNING
                        )
                        # 上传失败则回退到本地加载
                    msg = self.session.prompt
                    for handler in self.input_handler:
                        msg, _ = handler(msg, self)
                    self.session.prompt = f"{self.session.prompt}\n\n以下是历史类似问题的执行经验，可参考：\n{load_methodology(msg, self.get_tool_registry())}"
                else:
                    if self.files:
                        self.session.prompt = f"{self.session.prompt}\n\n上传的文件包含历史对话信息和方法论文件，可以从中获取一些经验信息。"
                    else:
                        self.session.prompt = f"{self.session.prompt}\n\n上传的文件包含历史对话信息，可以从中获取一些经验信息。"
            elif self.files:
                if not self.model.upload_files(self.files):
                    PrettyOutput.print(
                        "文件上传失败，将忽略文件列表", OutputType.WARNING
                    )
                else:
                    self.session.prompt = f"{self.session.prompt}\n\n上传的文件包含历史对话信息，可以从中获取一些经验信息。"
        else:
            if self.files:
                PrettyOutput.print("不支持上传文件，将忽略文件列表", OutputType.WARNING)
            if self.use_methodology:
                msg = self.session.prompt
                for handler in self.input_handler:
                    msg, _ = handler(msg, self)
                self.session.prompt = f"{self.session.prompt}\n\n以下是历史类似问题的执行经验，可参考：\n{load_methodology(msg, self.get_tool_registry())}"

        self.first = False

    def clear_history(self):
        """
        Clears conversation history by delegating to the session manager.
        """
        self.session.clear_history()



# -*- coding: utf-8 -*-
import json
import os
from typing import Dict, Generator, List, Tuple

from openai import OpenAI

from jarvis.jarvis_platform.base import BasePlatform
from jarvis.jarvis_utils.output import OutputType, PrettyOutput


class OpenAIModel(BasePlatform):

    def __init__(self):
        """
        Initialize OpenAI model
        """
        super().__init__()
        self.system_message = ""
        self.api_key = os.getenv("OPENAI_API_KEY")
        if not self.api_key:
            PrettyOutput.print("OPENAI_API_KEY 未设置", OutputType.WARNING)

        self.base_url = os.getenv("OPENAI_API_BASE", "https://api.openai.com/v1")
        self.model_name = os.getenv("JARVIS_MODEL") or "gpt-4o"

        self.client = OpenAI(api_key=self.api_key, base_url=self.base_url)
        self.messages: List[Dict[str, str]] = []
        self.system_message = ""

    def upload_files(self, file_list: List[str]) -> bool:
        """
        上传文件到OpenAI平台

        参数:
            file_list: 需要上传的文件路径列表

        返回:
            bool: 上传是否成功 (当前实现始终返回False)
        """
        return False

    def get_model_list(self) -> List[Tuple[str, str]]:
        """
        获取可用的OpenAI模型列表

        返回:
            List[Tuple[str, str]]: 模型ID和名称的元组列表

        异常:
            当API调用失败时会打印错误信息并返回空列表
        """
        try:
            models = self.client.models.list()
            model_list = []
            for model in models:
                model_list.append((model.id, model.id))
            return model_list
        except Exception as e:
            PrettyOutput.print(f"获取模型列表失败：{str(e)}", OutputType.ERROR)
            return []

    def set_model_name(self, model_name: str):
        """
        设置当前使用的模型名称

        参数:
            model_name: 要设置的模型名称
        """

        self.model_name = model_name

    def set_system_prompt(self, message: str):
        """
        设置系统消息(角色设定)

        参数:
            message: 系统消息内容

        说明:
            设置后会立即添加到消息历史中
        """
        self.system_message = message
        self.messages.append({"role": "system", "content": self.system_message})

    def chat(self, message: str) -> Generator[str, None, None]:
        """
        执行对话并返回生成器

        参数:
            message: 用户输入的消息内容

        返回:
            Generator[str, None, None]: 生成器，逐块返回AI响应内容

        异常:
            当API调用失败时会抛出异常并打印错误信息
        """
        try:
            # Add user message to history
            self.messages.append({"role": "user", "content": message})

            response = self.client.chat.completions.create(
                model=self.model_name,  # Use the configured model name
                messages=self.messages,  # type: ignore
                stream=True,
            )  # type: ignore

            full_response = ""
            for chunk in response:
                if chunk.choices and chunk.choices[0].delta.content:
                    text = chunk.choices[0].delta.content
                    full_response += text
                    yield text

            # Add assistant reply to history
            self.messages.append({"role": "assistant", "content": full_response})

            return None

        except Exception as e:
            PrettyOutput.print(f"对话失败：{str(e)}", OutputType.ERROR)
            raise Exception(f"Chat failed: {str(e)}")

    def name(self) -> str:
        """
        获取当前使用的模型名称

        返回:
            str: 当前配置的模型名称
        """
        return self.model_name

    @classmethod
    def platform_name(cls) -> str:
        """
        获取当前平台的名称

        返回:
            str: 当前平台的名称
        """
        return "openai"

    def delete_chat(self) -> bool:
        """
        删除当前对话历史

        返回:
            bool: 操作是否成功

        说明:
            如果设置了系统消息，会保留系统消息
        """
        if self.system_message:
            self.messages = [{"role": "system", "content": self.system_message}]
        else:
            self.messages = []
        return True

    def save(self, file_path: str) -> bool:
        """Save chat session to a file."""
        state = {
            "messages": self.messages,
            "model_name": self.model_name,
        }

        try:
            with open(file_path, "w", encoding="utf-8") as f:
                json.dump(state, f, ensure_ascii=False, indent=4)
            self._saved = True
            PrettyOutput.print(f"会话已成功保存到 {file_path}", OutputType.SUCCESS)
            return True
        except Exception as e:
            PrettyOutput.print(f"保存会话失败: {str(e)}", OutputType.ERROR)
            return False

    def restore(self, file_path: str) -> bool:
        """Restore chat session from a file."""
        try:
            with open(file_path, "r", encoding="utf-8") as f:
                state = json.load(f)

            self.messages = state.get("messages", [])
            self.model_name = state.get("model_name", "gpt-4o")
            # atexit.register(self.delete_chat)
            self._saved = True

            PrettyOutput.print(f"从 {file_path} 成功恢复会话", OutputType.SUCCESS)
            return True
        except FileNotFoundError:
            PrettyOutput.print(f"会话文件未找到: {file_path}", OutputType.ERROR)
            return False
        except Exception as e:
            PrettyOutput.print(f"恢复会话失败: {str(e)}", OutputType.ERROR)
            return False

    def support_web(self) -> bool:
        """
        检查是否支持网页访问功能

        返回:
            bool: 当前是否支持网页访问 (OpenAI平台始终返回False)
        """
        return False

    def support_upload_files(self) -> bool:
        """
        检查是否支持上传文件功能

        返回:
            bool: 当前是否支持上传文件 (OpenAI平台始终返回False)
        """
        return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Lint工具配置模块
"""

import os
from typing import Dict, List

import yaml  # type: ignore

from jarvis.jarvis_utils.config import get_data_dir

# 默认的lint工具配置
LINT_TOOLS = {
    # C/C++
    ".c": ["clang-tidy"],
    ".cpp": ["clang-tidy"],
    ".cc": ["clang-tidy"],
    ".cxx": ["clang-tidy"],
    ".h": ["clang-tidy"],
    ".hpp": ["clang-tidy"],
    ".hxx": ["clang-tidy"],
    ".inl": ["clang-tidy"],
    ".ipp": ["clang-tidy"],
    # Go
    ".go": ["go vet"],
    # Python
    ".py": ["black", "pylint", "mypy"],
    ".pyw": ["black", "pylint", "mypy"],
    ".pyi": ["black", "pylint", "mypy"],
    ".pyx": ["black", "pylint", "mypy"],
    ".pxd": ["black", "pylint", "mypy"],
    # Rust
    ".rs": ["cargo clippy", "rustfmt"],
    ".rlib": ["cargo clippy", "rustfmt"],
    # Java
    ".java": ["pmd"],
    ".class": ["pmd"],
    ".jar": ["pmd"],
    # JavaScript/TypeScript
    ".js": ["eslint"],
    ".mjs": ["eslint"],
    ".cjs": ["eslint"],
    ".jsx": ["eslint"],
    ".ts": ["eslint", "tsc"],
    ".tsx": ["eslint", "tsc"],
    ".cts": ["eslint", "tsc"],
    ".mts": ["eslint", "tsc"],
    # PHP
    ".php": ["phpstan"],
    ".phtml": ["phpstan"],
    ".php5": ["phpstan"],
    ".php7": ["phpstan"],
    ".phps": ["phpstan"],
    # Ruby
    ".rb": ["rubocop"],
    ".rake": ["rubocop"],
    ".gemspec": ["rubocop"],
    # Swift
    ".swift": ["swiftlint"],
    # Kotlin
    ".kt": ["ktlint"],
    ".kts": ["ktlint"],
    # C#
    ".cs": ["roslynator"],
    ".csx": ["roslynator"],
    # SQL
    ".sql": ["sqlfluff"],
    # Shell/Bash
    ".sh": ["shellcheck"],
    ".bash": ["shellcheck"],
    # HTML/CSS
    ".html": ["htmlhint"],
    ".htm": ["htmlhint"],
    ".xhtml": ["htmlhint"],
    ".css": ["stylelint"],
    ".scss": ["stylelint"],
    ".sass": ["stylelint"],
    ".less": ["stylelint"],
    # XML/JSON/YAML
    ".xml": ["xmllint"],
    ".xsd": ["xmllint"],
    ".dtd": ["xmllint"],
    ".tld": ["xmllint"],
    ".jsp": ["xmllint"],
    ".jspx": ["xmllint"],
    ".tag": ["xmllint"],
    ".tagx": ["xmllint"],
    ".json": ["jsonlint"],
    ".jsonl": ["jsonlint"],
    ".json5": ["jsonlint"],
    ".yaml": ["yamllint"],
    ".yml": ["yamllint"],
    # Markdown/Documentation
    ".md": ["markdownlint"],
    ".markdown": ["markdownlint"],
    ".rst": ["rstcheck"],
    ".adoc": ["asciidoctor-lint"],
    # Docker/Terraform/Makefile等无后缀文件
    "makefile": ["checkmake"],
    "dockerfile": ["hadolint"],
    "docker-compose.yml": ["hadolint"],
    "docker-compose.yaml": ["hadolint"],
    "jenkinsfile": ["jenkinsfile-linter"],
    "build": ["buildifier"],
    "workspace": ["buildifier"],
    ".bashrc": ["shellcheck"],
    ".bash_profile": ["shellcheck"],
    ".zshrc": ["shellcheck"],
    ".gitignore": ["git-lint"],
    ".editorconfig": ["editorconfig-checker"],
    ".eslintrc": ["eslint"],
    ".prettierrc": ["prettier"],
    "cmakelists.txt": ["cmake-format"],
    ".cmake": ["cmake-format"],
}


def load_lint_tools_config() -> Dict[str, List[str]]:
    """从yaml文件加载lint工具配置"""
    config_path = os.path.join(get_data_dir(), "lint_tools.yaml")
    if not os.path.exists(config_path):
        return {}

    with open(config_path, "r") as f:
        config = yaml.safe_load(f) or {}
        return {k.lower(): v for k, v in config.items()}  # 确保key是小写


# 合并默认配置和yaml配置
LINT_TOOLS.update(load_lint_tools_config())


def get_lint_tools(filename: str) -> List[str]:
    """
    根据文件扩展名或文件名获取对应的lint工具列表

    Args:
        file_extension_or_name: 文件扩展名(如'.py')或文件名(如'Makefile')

    Returns:
        对应的lint工具列表，如果找不到则返回空列表
    """
    filename = os.path.basename(filename)
    lint_tools = LINT_TOOLS.get(filename.lower(), [])
    if lint_tools:
        return lint_tools
    ext = os.path.splitext(filename)[1]
    return LINT_TOOLS.get(ext.lower(), [])



# -*- coding: utf-8 -*-
"""
文件编辑工具类

功能概述:
1. 提供精确的文件内容搜索和替换功能，支持多组修改
2. 支持单个文件的编辑操作，包括创建新文件
3. 实现原子操作：所有修改要么全部成功，要么全部回滚
4. 严格匹配控制：每个搜索文本必须且只能匹配一次

核心特性:
- 支持不存在的文件和空文件处理
- 自动创建所需目录结构
- 完善的错误处理和回滚机制
- 严格的格式保持要求
"""
from typing import Any, Dict

from jarvis.jarvis_agent.edit_file_handler import EditFileHandler


class FileSearchReplaceTool:
    name = "edit_file"
    description = """代码编辑工具，用于精确修改一个或多个文件

# 文件编辑工具使用指南

## 基本使用
1. 指定需要修改的文件路径（单个或多个）
2. 提供一组或多组修改，每个修改包含:
   - reason: 修改原因描述
   - SEARCH: 需要查找的原始代码(必须包含足够上下文)
   - REPLACE: 替换后的新代码
3. 工具会自动选择最适合的编辑模式

## 核心原则
1. **精准修改**: 只修改必要的代码部分，保持其他部分不变
2. **最小补丁原则**: 生成最小范围的补丁，包含必要的上下文
3. **唯一匹配**: 确保搜索文本在文件中唯一匹配
4. **格式保持**: 严格保持原始代码的格式风格
5. **部分成功**: 支持多个文件编辑，允许部分文件编辑成功

"""
    parameters = {
        "type": "object",
        "properties": {
            "files": {
                "type": "array",
                "description": "需要修改的文件路径列表",
                "items": {
                    "type": "object",
                    "properties": {
                        "path": {"type": "string", "description": "文件路径"},
                        "changes": {
                            "type": "array",
                            "description": "一组或多组修改，每个修改必须包含1-2行上下文用于精确定位",
                            "items": {
                                "type": "object",
                                "properties": {
                                    "reason": {
                                        "type": "string",
                                        "description": "修改的原因",
                                    },
                                    "SEARCH": {
                                        "type": "string",
                                        "description": "需要查找的原始代码",
                                    },
                                    "REPLACE": {
                                        "type": "string",
                                        "description": "替换后的新代码",
                                    },
                                },
                            },
                        },
                    },
                    "required": ["path", "changes"],
                },
            },
        },
        "required": ["files"],
    }

    def execute(self, args: Dict) -> Dict[str, Any]:
        """执行文件编辑操作，支持快速编辑和AI辅助编辑两种模式。

        主要功能:
        1. 处理多个文件的创建或修改，支持不存在的文件
        2. 每个文件独立处理，允许部分文件编辑成功
        3. 自动选择编辑模式(fast_edit或slow_edit)
        4. 保存修改前后的文件状态以便回滚
        5. 提供详细的执行状态输出

        参数:
            args: 包含以下键的字典:
                - files: 文件列表，每个文件包含(必填):
                    - path: 要修改的文件路径
                    - changes: 修改列表，每个修改包含:
                        - reason: 修改原因描述
                        - SEARCH: 需要查找的原始代码(必须包含足够上下文)
                        - REPLACE: 替换后的新代码

        返回:
            Dict[str, Any] 包含:
                - success: 是否至少有一个文件编辑成功(True/False)
                - stdout: 成功时的输出消息
                - stderr: 失败时的错误消息
                - results: 每个文件的处理结果列表

        异常处理:
        1. 捕获并记录文件操作异常
        2. 失败的修改尝试回滚到原始状态
        3. 新创建的文件在失败时会被删除
        4. 提供3次重试机制确保操作可靠性
        5. 支持大文件处理(自动上传到模型平台)

        实现细节:
        1. 优先尝试fast_edit模式
        2. 如果fast_edit失败，则尝试slow_edit模式
        3. 严格检查搜索文本的唯一匹配性
        4. 保持原始代码的格式风格
        """
        import os

        from jarvis.jarvis_utils.output import OutputType, PrettyOutput

        stdout_messages = []
        stderr_messages = []
        overall_success = False
        file_results = []

        for file_info in args["files"]:
            file_path = os.path.abspath(file_info["path"])
            changes = file_info["changes"]

            # 创建已处理文件变量，用于失败时回滚
            original_content = None
            processed = False
            file_success = True

            try:
                file_exists = os.path.exists(file_path)
                content = ""

                try:
                    # 如果文件存在，则读取内容
                    if file_exists:
                        with open(file_path, "r", encoding="utf-8") as f:
                            content = f.read()
                            original_content = content

                    print(f"⚙️ 正在处理文件 {file_path}...")
                    success, temp_content = EditFileHandler._fast_edit(
                        file_path, changes
                    )
                    if not success:
                        print(f"❌ 文件 {file_path} 处理失败")
                        file_results.append(
                            {
                                "file": file_path,
                                "success": False,
                                "stdout": "",
                                "stderr": temp_content,
                            }
                        )
                        continue

                    print(f"✅ 文件 {file_path} 内容生成完成")

                    # 只有当所有替换操作都成功时，才写回文件
                    if success and (
                        temp_content != original_content or not file_exists
                    ):
                        # 确保目录存在
                        os.makedirs(
                            os.path.dirname(os.path.abspath(file_path)), exist_ok=True
                        )

                        with open(file_path, "w", encoding="utf-8") as f:
                            f.write(temp_content)

                        processed = True

                        action = "创建并写入" if not file_exists else "成功修改"
                        stdout_message = f"文件 {file_path} {action} 完成"
                        stdout_messages.append(stdout_message)
                        PrettyOutput.print(stdout_message, OutputType.SUCCESS)
                        overall_success = True

                        file_results.append(
                            {
                                "file": file_path,
                                "success": True,
                                "stdout": stdout_message,
                                "stderr": "",
                            }
                        )

                except Exception as e:
                    stderr_message = f"处理文件 {file_path} 时出错: {str(e)}"
                    stderr_messages.append(stderr_message)
                    PrettyOutput.print(stderr_message, OutputType.WARNING)
                    file_success = False
                    file_results.append(
                        {
                            "file": file_path,
                            "success": False,
                            "stdout": "",
                            "stderr": stderr_message,
                        }
                    )

            except Exception as e:
                error_msg = f"文件搜索替换操作失败: {str(e)}"
                PrettyOutput.print(error_msg, OutputType.WARNING)

                # 如果有已修改的文件，尝试回滚
                if processed:
                    rollback_message = "操作失败，正在回滚修改..."
                    stderr_messages.append(rollback_message)
                    PrettyOutput.print(rollback_message, OutputType.WARNING)

                    try:
                        if original_content is None:
                            # 如果是新创建的文件，则删除
                            if os.path.exists(file_path):
                                os.remove(file_path)
                            stderr_messages.append(f"已删除新创建的文件: {file_path}")
                        else:
                            # 如果是修改的文件，则恢复原内容
                            with open(file_path, "w", encoding="utf-8") as f:
                                f.write(original_content)
                            stderr_messages.append(f"已回滚文件: {file_path}")
                    except:
                        stderr_messages.append(f"回滚文件失败: {file_path}")

                file_results.append(
                    {
                        "file": file_path,
                        "success": False,
                        "stdout": "",
                        "stderr": error_msg,
                    }
                )

        # 整合所有错误信息到stderr
        all_stderr = []
        for result in file_results:
            if not result["success"]:
                all_stderr.append(f"文件 {result['file']} 处理失败: {result['stderr']}")

        return {
            "success": overall_success,
            "stdout": "\n".join(stdout_messages) if overall_success else "",
            "stderr": "\n".join(all_stderr) if not overall_success else "",
        }



# -*- coding: utf-8 -*-
import os
from pathlib import Path
from typing import List

from jarvis.jarvis_utils.output import OutputType, PrettyOutput

# 设置tiktoken缓存目录
script_dir = Path(__file__).parent
tiktoken_cache_dir = script_dir / "../jarvis_data/tiktoken"
os.makedirs(tiktoken_cache_dir, exist_ok=True)
os.environ["TIKTOKEN_CACHE_DIR"] = str(tiktoken_cache_dir.absolute())


def get_context_token_count(text: str) -> int:
    """使用tiktoken获取文本的token数量。

    参数：
        text: 要计算token的输入文本

    返回：
        int: 文本中的token数量
    """
    try:
        import tiktoken

        encoding = tiktoken.get_encoding("cl100k_base")
        return len(encoding.encode(text))
    except Exception as e:
        PrettyOutput.print(f"计算token失败: {str(e)}", OutputType.WARNING)
        return len(text) // 4  # 每个token大约4个字符的粗略估计


def split_text_into_chunks(
    text: str, max_length: int = 512, min_length: int = 50
) -> List[str]:
    """将文本分割成块，基于token数量进行切割。

    参数：
        text: 要分割的输入文本
        max_length: 每个块的最大token数量
        min_length: 每个块的最小token数量（除了最后一块可能较短）

    返回：
        List[str]: 文本块列表，每个块的token数量不超过max_length且不小于min_length
    """
    if not text:
        return []

    try:
        chunks = []
        current_chunk = ""
        current_tokens = 0

        # 按较大的块处理文本，避免破坏token边界
        chunk_size = 50  # 每次处理50个字符
        for i in range(0, len(text), chunk_size):
            chunk = text[i : i + chunk_size]
            chunk_tokens = get_context_token_count(chunk)

            # 如果当前块加上新块会超过最大长度，且当前块已经达到最小长度，则保存当前块
            if (
                current_tokens + chunk_tokens > max_length
                and current_tokens >= min_length
            ):
                chunks.append(current_chunk)
                current_chunk = chunk
                current_tokens = chunk_tokens
            else:
                current_chunk += chunk
                current_tokens += chunk_tokens

        # 处理最后一个块
        if current_chunk:
            chunks.append(current_chunk)  # 直接添加最后一个块，无论长度如何

        return chunks

    except Exception as e:
        PrettyOutput.print(f"文本分割失败: {str(e)}", OutputType.WARNING)
        # 发生错误时回退到简单的字符分割
        return [text[i : i + max_length] for i in range(0, len(text), max_length)]



import os
import re
from typing import Any, Dict, List, Tuple

from jarvis.jarvis_agent.output_handler import OutputHandler
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.tag import ct, ot


class EditFileHandler(OutputHandler):
    def __init__(self):
        self.patch_pattern = re.compile(
            ot("PATCH file=(?:'([^']+)'|\"([^\"]+)\"|([^>]+))") + r"\s*"
            r"(?:"
            + ot("DIFF")
            + r"\s*"
            + ot("SEARCH")
            + r"(.*?)"
            + ct("SEARCH")
            + r"\s*"
            + ot("REPLACE")
            + r"(.*?)"
            + ct("REPLACE")
            + r"\s*"
            + ct("DIFF")
            + r"\s*)+"
            + ct("PATCH"),
            re.DOTALL,
        )
        self.diff_pattern = re.compile(
            ot("DIFF")
            + r"\s*"
            + ot("SEARCH")
            + r"(.*?)"
            + ct("SEARCH")
            + r"\s*"
            + ot("REPLACE")
            + r"(.*?)"
            + ct("REPLACE")
            + r"\s*"
            + ct("DIFF"),
            re.DOTALL,
        )

    def handle(self, response: str, agent: Any) -> Tuple[bool, str]:
        """处理文件编辑响应

        Args:
            response: 包含文件编辑指令的响应字符串
            agent: 执行处理的agent实例

        Returns:
            Tuple[bool, str]: 返回处理结果元组，第一个元素表示是否处理成功，第二个元素为处理结果汇总字符串
        """
        patches = self._parse_patches(response)
        if not patches:
            return False, "未找到有效的文件编辑指令"

        results = []

        for file_path, diffs in patches.items():
            file_path = os.path.abspath(file_path)
            file_patches = [
                {"SEARCH": diff["SEARCH"], "REPLACE": diff["REPLACE"]} for diff in diffs
            ]

            print(f"📝 正在处理文件 {file_path}...")
            success, result = self._fast_edit(file_path, file_patches)

            if success:
                results.append(f"✅ 文件 {file_path} 修改成功")
            else:
                results.append(f"❌ 文件 {file_path} 修改失败: {result}")

        summary = "\n".join(results)
        return False, summary

    def can_handle(self, response: str) -> bool:
        """判断是否能处理给定的响应

        Args:
            response: 需要判断的响应字符串

        Returns:
            bool: 返回是否能处理该响应
        """
        return bool(self.patch_pattern.search(response))

    def prompt(self) -> str:
        """获取处理器的提示信息

        Returns:
            str: 返回处理器的提示字符串
        """
        return f"""文件编辑指令格式：
{ot("PATCH file=文件路径")}
{ot("DIFF")}
{ot("SEARCH")}原始代码{ct("SEARCH")}
{ot("REPLACE")}新代码{ct("REPLACE")}
{ct("DIFF")}
{ct("PATCH")}

可以返回多个PATCH块用于同时修改多个文件
每个PATCH块可以包含多个DIFF块，每个DIFF块包含一组搜索和替换内容。
搜索文本必须能在文件中唯一匹配，否则编辑将失败。"""

    def name(self) -> str:
        """获取处理器的名称

        Returns:
            str: 返回处理器的名称字符串
        """
        return "PATCH"

    def _parse_patches(self, response: str) -> Dict[str, List[Dict[str, str]]]:
        """解析响应中的补丁信息

        该方法使用正则表达式从响应文本中提取文件编辑指令(PATCH块)，
        每个PATCH块可以包含多个DIFF块，每个DIFF块包含一组搜索和替换内容。
        解析后会返回一个字典，键是文件路径，值是该文件对应的补丁列表。
        如果同一个文件路径出现多次，会将所有DIFF块合并到一起。

        Args:
            response: 包含补丁信息的响应字符串，格式应符合PATCH指令规范

        Returns:
            Dict[str, List[Dict[str, str]]]:
                返回解析后的补丁信息字典，结构为:
                {
                    "文件路径1": [
                        {"SEARCH": "搜索文本1", "REPLACE": "替换文本1"},
                        {"SEARCH": "搜索文本2", "REPLACE": "替换文本2"}
                    ],
                    "文件路径2": [...]
                }
        """
        patches: Dict[str, List[Dict[str, str]]] = {}
        for match in self.patch_pattern.finditer(response):
            # Get the file path from the appropriate capture group
            file_path = match.group(1) or match.group(2) or match.group(3)
            diffs = []
            for diff_match in self.diff_pattern.finditer(match.group(0)):
                # 完全保留原始格式（包括所有空白和换行）
                diffs.append(
                    {
                        "SEARCH": diff_match.group(1),  # 原始SEARCH内容
                        "REPLACE": diff_match.group(2),  # 原始REPLACE内容
                    }
                )
            if diffs:
                if file_path in patches:
                    patches[file_path].extend(diffs)
                else:
                    patches[file_path] = diffs
        return patches

    @staticmethod
    def _fast_edit(file_path: str, patches: List[Dict[str, str]]) -> Tuple[bool, str]:
        """快速应用补丁到文件

        该方法直接尝试将补丁应用到目标文件，适用于简单、明确的修改场景。
        特点：
        1. 直接进行字符串替换，效率高
        2. 会自动处理缩进问题，尝试匹配不同缩进级别的代码
        3. 确保搜索文本在文件中唯一匹配
        4. 如果部分补丁失败，会继续应用剩余补丁，并报告失败信息

        Args:
            file_path: 要修改的文件路径，支持绝对路径和相对路径
            patches: 补丁列表，每个补丁包含search(搜索文本)和replace(替换文本)

        Returns:
            Tuple[bool, str]:
                返回处理结果元组，第一个元素表示是否所有补丁都成功应用，
                第二个元素为结果信息，全部成功时为修改后的文件内容，部分或全部失败时为错误信息
        """
        try:
            # 确保目录存在
            os.makedirs(os.path.dirname(file_path), exist_ok=True)

            # 读取原始文件内容
            file_content = ""
            if os.path.exists(file_path):
                with open(file_path, "r", encoding="utf-8") as f:
                    file_content = f.read()

            # 应用所有补丁
            modified_content = file_content
            patch_count = 0
            failed_patches: List[Dict[str, Any]] = []
            successful_patches = 0

            for patch in patches:
                patch_count += 1
                search_text = patch["SEARCH"]
                replace_text = patch["REPLACE"]

                # 精确匹配搜索文本（保留原始换行和空格）
                exact_search = search_text
                found = False

                if exact_search in modified_content:
                    # 直接执行替换（保留所有原始格式）
                    modified_content = modified_content.replace(
                        exact_search, replace_text
                    )
                    print(f"✅ 补丁 #{patch_count} 应用成功")
                    found = True
                else:
                    # 如果匹配不到，并且search与replace块的首尾都是换行，尝试去掉第一个和最后一个换行
                    if (
                        search_text.startswith("\n")
                        and search_text.endswith("\n")
                        and replace_text.startswith("\n")
                        and replace_text.endswith("\n")
                    ):
                        stripped_search = search_text[1:-1]
                        stripped_replace = replace_text[1:-1]
                        if stripped_search in modified_content:
                            modified_content = modified_content.replace(
                                stripped_search, stripped_replace
                            )
                            print(f"✅ 补丁 #{patch_count} 应用成功 (自动去除首尾换行)")
                            found = True

                    if not found:
                        # 尝试增加缩进重试
                        current_search = search_text
                        current_replace = replace_text
                        if (
                            current_search.startswith("\n")
                            and current_search.endswith("\n")
                            and current_replace.startswith("\n")
                            and current_replace.endswith("\n")
                        ):
                            current_search = current_search[1:-1]
                            current_replace = current_replace[1:-1]

                        for space_count in range(1, 17):
                            indented_search = "\n".join(
                                " " * space_count + line if line.strip() else line
                                for line in current_search.split("\n")
                            )
                            indented_replace = "\n".join(
                                " " * space_count + line if line.strip() else line
                                for line in current_replace.split("\n")
                            )
                            if indented_search in modified_content:
                                modified_content = modified_content.replace(
                                    indented_search, indented_replace
                                )
                                print(
                                    f"✅ 补丁 #{patch_count} 应用成功 (自动增加 {space_count} 个空格缩进)"
                                )
                                found = True
                                break

                if found:
                    successful_patches += 1
                else:
                    error_msg = "搜索文本在文件中不存在"
                    PrettyOutput.print(
                        f"{error_msg}：\n{search_text}",
                        output_type=OutputType.WARNING,
                    )
                    failed_patches.append({"patch": patch, "error": error_msg})

            # 写入修改后的内容
            with open(file_path, "w", encoding="utf-8") as f:
                f.write(modified_content)

            if failed_patches:
                error_details = [
                    f"  - 失败的补丁: \n{p['patch']['SEARCH']}\n    错误: {p['error']}"
                    for p in failed_patches
                ]
                summary = (
                    f"文件 {file_path} 修改部分成功。\n"
                    f"成功: {successful_patches}/{patch_count}, "
                    f"失败: {len(failed_patches)}/{patch_count}.\n"
                    f"失败详情:\n" + "\n".join(error_details)
                )
                print(f"❌ {summary}")
                return False, summary

            print(f"✅ 文件 {file_path} 修改完成，应用了 {patch_count} 个补丁")
            return True, modified_content

        except Exception as e:
            print(f"❌ 文件修改失败: {str(e)}")
            return False, f"文件修改失败: {str(e)}"



# -*- coding: utf-8 -*-
import re
import sys
from typing import Any, Tuple

from jarvis.jarvis_utils.config import get_replace_map


def builtin_input_handler(user_input: str, agent_: Any) -> Tuple[str, bool]:
    """
    处理内置的特殊输入标记，并追加相应的提示词

    参数：
        user_input: 用户输入
        agent: 代理对象

    返回：
        Tuple[str, bool]: 处理后的输入和是否需要进一步处理
    """
    from jarvis.jarvis_agent import Agent

    agent: Agent = agent_
    # 查找特殊标记
    special_tags = re.findall(r"'<([^>]+)>'", user_input)

    if not special_tags:
        return user_input, False

    # 获取替换映射表
    replace_map = get_replace_map()
    # 处理每个标记
    for tag in special_tags:
        # 优先处理特殊标记
        if tag == "Summary":
            agent._summarize_and_clear_history()
            return "", True
        elif tag == "Clear":
            agent.clear()
            return "", True
        elif tag == "ToolUsage":
            agent.set_addon_prompt(agent.get_tool_usage_prompt())
            return "", False
        elif tag == "ReloadConfig":
            from jarvis.jarvis_utils.utils import load_config

            load_config()
            return "", True
        elif tag == "SaveSession":
            if agent.save_session():
                from jarvis.jarvis_utils.output import OutputType, PrettyOutput

                PrettyOutput.print("会话已成功保存。正在退出...", OutputType.SUCCESS)
                sys.exit(0)
            else:
                from jarvis.jarvis_utils.output import OutputType, PrettyOutput

                PrettyOutput.print("保存会话失败。", OutputType.ERROR)
            return "", True

        processed_tag = set()
        add_on_prompt = ""

        # 处理普通替换标记
        if tag in replace_map:
            processed_tag.add(tag)
            if (
                "append" in replace_map[tag]
                and replace_map[tag]["append"]
                and tag not in processed_tag
            ):
                user_input = user_input.replace(f"'<{tag}>'", "")
                add_on_prompt += replace_map[tag]["template"] + "\n"
            else:
                user_input = user_input.replace(
                    f"'<{tag}>'", replace_map[tag]["template"]
                )

        agent.set_addon_prompt(add_on_prompt)

    return user_input, False



import os
import sys
from pathlib import Path
from typing import Optional, List, Literal, cast, Tuple
import mimetypes

import pathspec  # type: ignore
import typer
from langchain.docstore.document import Document
from langchain_community.document_loaders import (
    TextLoader,
    UnstructuredMarkdownLoader,
)
from langchain_core.document_loaders.base import BaseLoader
from rich.markdown import Markdown

from jarvis.jarvis_utils.utils import init_env
from jarvis.jarvis_utils.config import (
    get_rag_embedding_model,
    get_rag_use_bm25,
    get_rag_use_rerank,
)


def is_likely_text_file(file_path: Path) -> bool:
    """
    通过读取文件开头部分，检查文件是否可能为文本文件。
    此方法可以避免将大型二进制文件加载到内存中。
    """
    try:
        # 启发式方法1：检查MIME类型（如果可用）
        mime_type, _ = mimetypes.guess_type(file_path)
        if mime_type and mime_type.startswith("text/"):
            return True
        if mime_type and any(x in mime_type for x in ["json", "xml", "javascript"]):
            return True

        # 启发式方法2：检查文件的前几KB中是否包含空字节
        with open(file_path, "rb") as f:
            chunk = f.read(4096)  # 读取前4KB
            if b"\x00" in chunk:
                return False  # 空字节是二进制文件的强指示符
        return True
    except Exception:
        return False


# 确保项目根目录在Python路径中，以允许绝对导入
# 这使得脚本可以作为模块运行。
_project_root = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "..", "..", "..")
)
if _project_root not in sys.path:
    sys.path.insert(0, _project_root)

from jarvis.jarvis_platform.base import BasePlatform
from jarvis.jarvis_platform.registry import PlatformRegistry
from jarvis.jarvis_rag.llm_interface import LLMInterface
from jarvis.jarvis_rag.rag_pipeline import JarvisRAGPipeline

app = typer.Typer(
    name="jarvis-rag",
    help="一个与Jarvis RAG框架交互的命令行工具。",
    add_completion=False,
)


class _CustomPlatformLLM(LLMInterface):
    """一个简单的包装器，使BasePlatform实例与LLMInterface兼容。"""

    def __init__(self, platform: BasePlatform):
        self.platform = platform
        print(f"✅ 使用自定义LLM: 平台='{platform.platform_name()}', 模型='{platform.name()}'")

    def generate(self, prompt: str, **kwargs) -> str:
        return self.platform.chat_until_success(prompt)


def _create_custom_llm(platform_name: str, model_name: str) -> Optional[LLMInterface]:
    """从指定的平台和模型创建LLM接口。"""
    if not platform_name or not model_name:
        return None
    try:
        registry = PlatformRegistry.get_global_platform_registry()
        platform_instance = registry.create_platform(platform_name)
        if not platform_instance:
            print(f"❌ 错误: 平台 '{platform_name}' 未找到。")
            return None
        platform_instance.set_model_name(model_name)
        platform_instance.set_suppress_output(True)
        return _CustomPlatformLLM(platform_instance)
    except Exception as e:
        print(f"❌ 创建自定义LLM时出错: {e}")
        return None


def _load_ragignore_spec() -> Tuple[Optional[pathspec.PathSpec], Optional[Path]]:
    """
    从项目根目录加载忽略模式。
    首先查找 `.jarvis/rag/.ragignore`，如果未找到，则回退到 `.gitignore`。
    """
    project_root_path = Path(_project_root)
    ragignore_file = project_root_path / ".jarvis" / "rag" / ".ragignore"
    gitignore_file = project_root_path / ".gitignore"

    ignore_file_to_use = None
    if ragignore_file.is_file():
        ignore_file_to_use = ragignore_file
    elif gitignore_file.is_file():
        ignore_file_to_use = gitignore_file

    if ignore_file_to_use:
        try:
            with open(ignore_file_to_use, "r", encoding="utf-8") as f:
                patterns = f.read().splitlines()
            spec = pathspec.PathSpec.from_lines("gitwildmatch", patterns)
            print(f"✅ 加载忽略规则: {ignore_file_to_use}")
            return spec, project_root_path
        except Exception as e:
            print(f"⚠️ 加载 {ignore_file_to_use.name} 文件失败: {e}")

    return None, None


@app.command(
    "add",
    help="从文件、目录或glob模式（例如 'src/**/*.py'）添加文档。",
)
def add_documents(
    paths: List[Path] = typer.Argument(
        ...,
        help="文件/目录路径或glob模式。支持Shell扩展。",
    ),
    collection_name: str = typer.Option(
        "jarvis_rag_collection",
        "--collection",
        "-c",
        help="向量数据库中集合的名称。",
    ),
    embedding_model: Optional[str] = typer.Option(
        None,
        "--embedding-model",
        "-e",
        help="嵌入模型的名称。覆盖全局配置。",
    ),
    db_path: Optional[Path] = typer.Option(None, "--db-path", help="向量数据库的路径。覆盖全局配置。"),
    batch_size: int = typer.Option(
        500,
        "--batch-size",
        "-b",
        help="单个批次中要处理的文档数。",
    ),
):
    """从不同来源向RAG知识库添加文档。"""
    files_to_process = set()

    for path_str in paths:
        # Typer的List[Path]可能不会扩展glob，所以我们手动处理
        from glob import glob

        expanded_paths = glob(str(path_str), recursive=True)

        for p_str in expanded_paths:
            path = Path(p_str)
            if not path.exists():
                continue

            if path.is_dir():
                print(f"🔍 正在扫描目录: {path}")
                for item in path.rglob("*"):
                    if item.is_file() and is_likely_text_file(item):
                        files_to_process.add(item)
            elif path.is_file():
                if is_likely_text_file(path):
                    files_to_process.add(path)
                else:
                    print(f"⚠️ 跳过可能的二进制文件: {path}")

    if not files_to_process:
        print("⚠️ 在指定路径中未找到任何文本文件。")
        return

    # 使用 .ragignore 过滤文件
    ragignore_spec, ragignore_root = _load_ragignore_spec()
    if ragignore_spec and ragignore_root:
        initial_count = len(files_to_process)
        retained_files = set()
        for file_path in files_to_process:
            try:
                # 将文件路径解析为绝对路径以确保正确比较
                resolved_path = file_path.resolve()
                relative_path = str(resolved_path.relative_to(ragignore_root))
                if not ragignore_spec.match_file(relative_path):
                    retained_files.add(file_path)
            except ValueError:
                # 文件不在项目根目录下，保留它
                retained_files.add(file_path)

        ignored_count = initial_count - len(retained_files)
        if ignored_count > 0:
            print(f"ℹ️ 根据 .ragignore 规则过滤掉 {ignored_count} 个文件。")
        files_to_process = retained_files

    if not files_to_process:
        print("⚠️ 所有找到的文本文件都被忽略规则过滤掉了。")
        return

    print(f"✅ 发现 {len(files_to_process)} 个独立文件待处理。")

    try:
        pipeline = JarvisRAGPipeline(
            embedding_model=embedding_model,
            db_path=str(db_path) if db_path else None,
            collection_name=collection_name,
        )

        docs_batch: List[Document] = []
        total_docs_added = 0
        loader: BaseLoader

        sorted_files = sorted(list(files_to_process))
        total_files = len(sorted_files)

        for i, file_path in enumerate(sorted_files):
            try:
                if file_path.suffix.lower() == ".md":
                    loader = UnstructuredMarkdownLoader(str(file_path))
                else:  # 对.txt和所有代码文件默认使用TextLoader
                    loader = TextLoader(str(file_path), encoding="utf-8")

                docs_batch.extend(loader.load())
                print(f"✅ 已加载: {file_path} (文件 {i + 1}/{total_files})")
            except Exception as e:
                print(f"⚠️ 加载失败 {file_path}: {e}")

            # 当批处理已满或是最后一个文件时处理批处理
            if docs_batch and (len(docs_batch) >= batch_size or (i + 1) == total_files):
                print(f"⚙️ 正在处理批次，包含 {len(docs_batch)} 个文档...")
                pipeline.add_documents(docs_batch)
                total_docs_added += len(docs_batch)
                print(f"✅ 成功添加 {len(docs_batch)} 个文档。")
                docs_batch = []  # 清空批处理

        if total_docs_added == 0:
            print("❌ 未能成功加载任何文档。")
            raise typer.Exit(code=1)

        print(f"✅ 成功将 {total_docs_added} 个文档的内容添加至集合 '{collection_name}'。")

    except Exception as e:
        print(f"❌ 发生严重错误: {e}")
        raise typer.Exit(code=1)


@app.command("list-docs", help="列出知识库中所有唯一的文档。")
def list_documents(
    collection_name: str = typer.Option(
        "jarvis_rag_collection",
        "--collection",
        "-c",
        help="向量数据库中集合的名称。",
    ),
    db_path: Optional[Path] = typer.Option(None, "--db-path", help="向量数据库的路径。覆盖全局配置。"),
):
    """列出指定集合中的所有唯一文档。"""
    try:
        pipeline = JarvisRAGPipeline(
            db_path=str(db_path) if db_path else None,
            collection_name=collection_name,
        )

        collection = pipeline._get_collection()
        results = collection.get()  # 获取集合中的所有项目

        if not results or not results["metadatas"]:
            print("ℹ️ 知识库中没有找到任何文档。")
            return

        # 从元数据中提取唯一的源文件路径
        sources = set()
        for metadata in results["metadatas"]:
            if metadata:
                source = metadata.get("source")
                if isinstance(source, str):
                    sources.add(source)

        if not sources:
            print("ℹ️ 知识库中没有找到任何带有源信息的文档。")
            return

        print(f"📚 知识库 '{collection_name}' 中共有 {len(sources)} 个独立文档:")
        for i, source in enumerate(sorted(list(sources)), 1):
            print(f"  {i}. {source}")

    except Exception as e:
        print(f"❌ 发生错误: {e}")
        raise typer.Exit(code=1)


@app.command("retrieve", help="仅从知识库检索相关文档，不生成答案。")
def retrieve(
    question: str = typer.Argument(..., help="要提出的问题。"),
    collection_name: str = typer.Option(
        "jarvis_rag_collection",
        "--collection",
        "-c",
        help="向量数据库中集合的名称。",
    ),
    embedding_model: Optional[str] = typer.Option(
        None,
        "--embedding-model",
        "-e",
        help="嵌入模型的名称。覆盖全局配置。",
    ),
    db_path: Optional[Path] = typer.Option(None, "--db-path", help="向量数据库的路径。覆盖全局配置。"),
    n_results: int = typer.Option(5, "--top-n", help="要检索的文档数量。"),
):
    """仅从RAG知识库检索文档并打印结果。"""
    try:
        # 如果未在命令行中指定，则从配置中加载RAG设置
        final_embedding_model = embedding_model or get_rag_embedding_model()
        use_bm25 = get_rag_use_bm25()
        use_rerank = get_rag_use_rerank()

        pipeline = JarvisRAGPipeline(
            embedding_model=final_embedding_model,
            db_path=str(db_path) if db_path else None,
            collection_name=collection_name,
            use_bm25=use_bm25,
            use_rerank=use_rerank,
        )

        print(f"🤔 正在为问题检索文档: '{question}'")
        retrieved_docs = pipeline.retrieve_only(question, n_results=n_results)

        if not retrieved_docs:
            print("ℹ️ 未找到相关文档。")
            return

        print(f"✅ 成功检索到 {len(retrieved_docs)} 个文档:")
        from jarvis.jarvis_utils.globals import console

        for i, doc in enumerate(retrieved_docs, 1):
            source = doc.metadata.get("source", "未知来源")
            content = doc.page_content
            panel_title = f"文档 {i} | 来源: {source}"
            console.print(
                f"\n[bold magenta]{panel_title}[/bold magenta]"
            )
            console.print(Markdown(f"```\n{content}\n```"))

    except Exception as e:
        print(f"❌ 发生错误: {e}")
        raise typer.Exit(code=1)


@app.command("query", help="向知识库提问。")
def query(
    question: str = typer.Argument(..., help="要提出的问题。"),
    collection_name: str = typer.Option(
        "jarvis_rag_collection",
        "--collection",
        "-c",
        help="向量数据库中集合的名称。",
    ),
    embedding_model: Optional[str] = typer.Option(
        None,
        "--embedding-model",
        "-e",
        help="嵌入模型的名称。覆盖全局配置。",
    ),
    db_path: Optional[Path] = typer.Option(None, "--db-path", help="向量数据库的路径。覆盖全局配置。"),
    platform: Optional[str] = typer.Option(
        None,
        "--platform",
        "-p",
        help="为LLM指定平台名称。覆盖默认的思考模型。",
    ),
    model: Optional[str] = typer.Option(
        None,
        "--model",
        "-m",
        help="为LLM指定模型名称。需要 --platform。",
    ),
):
    """查询RAG知识库并打印答案。"""
    if model and not platform:
        print("❌ 错误: --model 需要指定 --platform。")
        raise typer.Exit(code=1)

    try:
        custom_llm = _create_custom_llm(platform, model) if platform and model else None
        if (platform or model) and not custom_llm:
            raise typer.Exit(code=1)

        # 如果未在命令行中指定，则从配置中加载RAG设置
        final_embedding_model = embedding_model or get_rag_embedding_model()
        use_bm25 = get_rag_use_bm25()
        use_rerank = get_rag_use_rerank()

        pipeline = JarvisRAGPipeline(
            llm=custom_llm,
            embedding_model=final_embedding_model,
            db_path=str(db_path) if db_path else None,
            collection_name=collection_name,
            use_bm25=use_bm25,
            use_rerank=use_rerank,
        )

        print(f"🤔 正在查询: '{question}'")
        answer = pipeline.query(question)

        print("💬 答案:")
        # 我们仍然可以使用 rich.markdown.Markdown，因为 PrettyOutput 底层使用了 rich
        from jarvis.jarvis_utils.globals import console

        console.print(Markdown(answer))

    except Exception as e:
        print(f"❌ 发生错误: {e}")
        raise typer.Exit(code=1)


_RAG_INSTALLED = False
try:
    import langchain  # noqa

    _RAG_INSTALLED = True
except ImportError:
    pass


def _check_rag_dependencies():
    if not _RAG_INSTALLED:
        print("❌ RAG依赖项未安装。" "请运行 'pip install \"jarvis-ai-assistant[rag]\"' 来使用此命令。")
        raise typer.Exit(code=1)


def main():
    _check_rag_dependencies()
    init_env(welcome_str="Jarvis RAG")
    app()



"""
Jarvis RAG 框架

一个灵活的RAG管道，具有可插拔的远程LLM和本地带缓存的嵌入模型。
"""

from .rag_pipeline import JarvisRAGPipeline
from .llm_interface import LLMInterface
from .embedding_manager import EmbeddingManager

__all__ = ["JarvisRAGPipeline", "LLMInterface", "EmbeddingManager"]



import sys
import json
from typing import Optional

import typer
from tabulate import tabulate

from jarvis.jarvis_tools.registry import ToolRegistry
from jarvis.jarvis_utils.output import OutputType, PrettyOutput
from jarvis.jarvis_utils.utils import init_env

app = typer.Typer(help="Jarvis 工具系统命令行界面")


@app.command("list")
def list_tools(
    as_json: bool = typer.Option(False, "--json", help="以JSON格式输出"),
    detailed: bool = typer.Option(False, "--detailed", help="显示详细信息"),
):
    """列出所有可用工具"""
    registry = ToolRegistry()
    tools = registry.get_all_tools()

    if as_json:
        if detailed:
            print(json.dumps(tools, indent=2, ensure_ascii=False))
        else:
            simple_tools = [
                {"name": t["name"], "description": t["description"]} for t in tools
            ]
            print(json.dumps(simple_tools, indent=2, ensure_ascii=False))
    else:
        PrettyOutput.section("可用工具列表", OutputType.SYSTEM)
        for tool in tools:
            print(f"\n✅ {tool['name']}")
            print(f"   描述: {tool['description']}")
            if detailed:
                print("   参数:")
                print(tool["parameters"])


@app.command("stat")
def stat_tools(as_json: bool = typer.Option(False, "--json", help="以JSON格式输出")):
    """显示工具调用统计信息"""
    registry = ToolRegistry()
    stats = registry._get_tool_stats()
    tools = registry.get_all_tools()

    table_data = []
    for tool in tools:
        name = tool["name"]
        count = stats.get(name, 0)
        table_data.append([name, count])

    table_data.sort(key=lambda x: x[1], reverse=True)

    if as_json:
        print(json.dumps(dict(table_data), indent=2))
    else:
        PrettyOutput.section("工具调用统计", OutputType.SYSTEM)
        print(tabulate(table_data, headers=["工具名称", "调用次数"], tablefmt="grid"))


@app.command("call")
def call_tool(
    tool_name: str = typer.Argument(..., help="要调用的工具名称"),
    args: Optional[str] = typer.Option(None, "--args", help="工具参数 (JSON格式)"),
    args_file: Optional[str] = typer.Option(
        None, "--args-file", help="从文件加载工具参数 (JSON格式)"
    ),
):
    """调用指定工具"""
    registry = ToolRegistry()
    tool_obj = registry.get_tool(tool_name)

    if not tool_obj:
        PrettyOutput.print(f"错误: 工具 '{tool_name}' 不存在", OutputType.ERROR)
        available_tools = ", ".join([t["name"] for t in registry.get_all_tools()])
        print(f"可用工具: {available_tools}")
        raise typer.Exit(code=1)

    tool_args = {}
    if args:
        try:
            tool_args = json.loads(args)
        except json.JSONDecodeError:
            PrettyOutput.print("错误: 参数必须是有效的JSON格式", OutputType.ERROR)
            raise typer.Exit(code=1)
    elif args_file:
        try:
            with open(args_file, "r", encoding="utf-8") as f:
                tool_args = json.load(f)
        except (json.JSONDecodeError, FileNotFoundError) as e:
            PrettyOutput.print(
                f"错误: 无法从文件加载参数: {str(e)}", OutputType.ERROR
            )
            raise typer.Exit(code=1)

    required_params = tool_obj.parameters.get("required", [])
    missing_params = [p for p in required_params if p not in tool_args]

    if missing_params:
        PrettyOutput.print(
            f"错误: 缺少必需参数: {', '.join(missing_params)}", OutputType.ERROR
        )
        print("\n参数说明:")
        params = tool_obj.parameters.get("properties", {})
        for param_name in required_params:
            param_info = params.get(param_name, {})
            desc = param_info.get("description", "无描述")
            print(f"  - {param_name}: {desc}")
        raise typer.Exit(code=1)

    result = registry.execute_tool(tool_name, tool_args)

    if result["success"]:
        PrettyOutput.section(f"工具 {tool_name} 执行成功", OutputType.SUCCESS)
    else:
        PrettyOutput.section(f"工具 {tool_name} 执行失败", OutputType.ERROR)

    if result.get("stdout"):
        print("\n输出:")
        print(result["stdout"])

    if result.get("stderr"):
        PrettyOutput.print("\n错误:", OutputType.ERROR)
        print(result["stderr"])

    if not result["success"]:
        raise typer.Exit(code=1)


def cli():
    """Typer application entry point"""
    init_env("欢迎使用 Jarvis-Tools，您的工具系统已准备就绪！")
    app()


def main():
    """Main entry point for the script"""
    cli()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""Jarvis Platform Manager Service Module.

This module provides an OpenAI-compatible API service for the Jarvis platform.
"""
import asyncio
import json
import os
import time
import uuid
from datetime import datetime
from typing import Any, Dict, List, Optional, Union

import uvicorn
from fastapi import FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import StreamingResponse
from pydantic import BaseModel, Field
from starlette.responses import JSONResponse, Response

from jarvis.jarvis_platform.registry import PlatformRegistry
from jarvis.jarvis_utils.output import OutputType, PrettyOutput


class ChatMessage(BaseModel):
    """Represents a chat message with role and content."""

    role: str
    content: str


class ChatCompletionRequest(BaseModel):
    """Request model for chat completion."""

    model: str
    messages: List[ChatMessage]
    stream: bool = False
    temperature: Optional[float] = None
    max_tokens: Optional[int] = None


class ChatCompletionChoice(BaseModel):
    """Represents a choice in chat completion response."""

    index: int
    message: ChatMessage
    finish_reason: str = "stop"


class ChatCompletionResponse(BaseModel):
    """Response model for chat completion."""

    id: str
    object: str = "chat.completion"
    created: int
    model: str
    choices: List[ChatCompletionChoice]
    usage: Dict[str, int] = Field(
        default_factory=lambda: {
            "prompt_tokens": 0,
            "completion_tokens": 0,
            "total_tokens": 0,
        }
    )


def start_service(
    host: str,
    port: int,
    default_platform: Optional[str] = None,
    default_model: Optional[str] = None,
) -> None:
    """Start OpenAI-compatible API server."""
    # Create logs directory if it doesn't exist
    logs_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "logs")
    os.makedirs(logs_dir, exist_ok=True)

    app = FastAPI(title="Jarvis API Server")

    # Add CORS middleware
    app.add_middleware(
        CORSMiddleware,
        allow_origins=["*"],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

    registry = PlatformRegistry.get_global_platform_registry()

    PrettyOutput.print(
        f"Starting Jarvis API server on {host}:{port}", OutputType.SUCCESS
    )
    PrettyOutput.print("This server provides an OpenAI-compatible API", OutputType.INFO)

    if default_platform and default_model:
        PrettyOutput.print(
            f"Default platform: {default_platform}, model: {default_model}",
            OutputType.INFO,
        )

    PrettyOutput.print("Available platforms:", OutputType.INFO)

    # Platform and model cache
    platform_instances: Dict[str, Any] = {}

    # Chat history storage
    chat_histories: Dict[str, Dict[str, Any]] = {}

    def get_platform_instance(platform_name: str, model_name: str) -> Any:
        """Get or create a platform instance."""
        key = f"{platform_name}:{model_name}"
        if key not in platform_instances:
            platform = registry.create_platform(platform_name)
            if not platform:
                raise HTTPException(
                    status_code=400, detail=f"Platform {platform_name} not found"
                )

            platform.set_model_name(model_name)
            platform_instances[key] = platform

        return platform_instances[key]

    def log_conversation(
        conversation_id: str,
        messages: List[Dict[str, str]],
        model: str,
        response: Optional[str] = None,
    ) -> None:
        """Log conversation to file in plain text format."""
        timestamp = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
        log_file = os.path.join(
            logs_dir, f"conversation_{conversation_id}_{timestamp}.txt"
        )

        with open(log_file, "w", encoding="utf-8", errors="ignore") as f:
            f.write(f"Conversation ID: {conversation_id}\n")
            f.write(f"Timestamp: {timestamp}\n")
            f.write(f"Model: {model}\n\n")
            f.write("Messages:\n")
            for message in messages:
                f.write(f"{message['role']}: {message['content']}\n")
            if response:
                f.write(f"\nResponse:\n{response}\n")

        PrettyOutput.print(f"Conversation logged to {log_file}", OutputType.INFO)

    @app.get("/v1/models")
    async def list_models() -> Dict[str, Any]:
        """List available models for the specified platform in OpenAI-compatible format."""
        model_list = []

        # Only get models for the currently set platform
        if default_platform:
            try:
                platform = registry.create_platform(default_platform)
                if platform:
                    models = platform.get_model_list()
                    if models:
                        for model_name, _ in models:
                            full_name = f"{default_platform}/{model_name}"
                            model_list.append(
                                {
                                    "id": full_name,
                                    "object": "model",
                                    "created": int(time.time()),
                                    "owned_by": default_platform,
                                }
                            )
            except Exception as exc:
                print(f"Error getting models for {default_platform}: {str(exc)}")

        # Return model list
        return {"object": "list", "data": model_list}

    @app.post("/v1/chat/completions", response_model=None)
    @app.options("/v1/chat/completions")
    async def create_chat_completion(
        request: ChatCompletionRequest,
    ) -> Response:
        """Create a chat completion in OpenAI-compatible format.

        Returns:
            Response: Either a JSONResponse or StreamingResponse depending on the request.
        """
        model = request.model
        messages = request.messages
        stream = request.stream

        # Generate a conversation ID if this is a new conversation
        conversation_id = str(uuid.uuid4())

        # Extract platform and model name
        if "/" in model:
            platform_name, model_name = model.split("/", 1)
        else:
            # Use default platform and model if not specified
            if default_platform and default_model:
                platform_name, model_name = default_platform, default_model
            else:
                platform_name, model_name = "oyi", model  # Default to OYI platform

        # Get platform instance
        platform = get_platform_instance(platform_name, model_name)

        # Convert messages to text format for the platform
        message_text = ""
        for msg in messages:
            role = msg.role
            content = msg.content

            if role == "system":
                message_text += f"System: {content}\n\n"
            elif role == "user":
                message_text += f"User: {content}\n\n"
            elif role == "assistant":
                message_text += f"Assistant: {content}\n\n"

        # Store messages in chat history
        chat_histories[conversation_id] = {
            "model": model,
            "messages": [{"role": m.role, "content": m.content} for m in messages],
        }

        # Log the conversation
        log_conversation(
            conversation_id,
            [{"role": m.role, "content": m.content} for m in messages],
            model,
        )

        if stream:
            # Return streaming response
            return StreamingResponse(
                stream_chat_response(platform, message_text, model),  # type: ignore
                media_type="text/event-stream",
            )

        # Get chat response
        try:
            response_text = platform.chat_until_success(message_text)

            # Create response in OpenAI format
            completion_id = f"chatcmpl-{str(uuid.uuid4())}"

            # Update chat history with response
            if conversation_id in chat_histories:
                chat_histories[conversation_id]["messages"].append(
                    {"role": "assistant", "content": response_text}
                )

            # Log the conversation with response
            log_conversation(
                conversation_id,
                chat_histories[conversation_id]["messages"],
                model,
                response_text,
            )

            return JSONResponse(
                {
                    "id": completion_id,
                    "object": "chat.completion",
                    "created": int(time.time()),
                    "model": model,
                    "choices": [
                        {
                            "index": 0,
                            "message": {"role": "assistant", "content": response_text},
                            "finish_reason": "stop",
                        }
                    ],
                    "usage": {
                        "prompt_tokens": len(message_text) // 4,
                        "completion_tokens": len(response_text) // 4,
                        "total_tokens": (len(message_text) + len(response_text)) // 4,
                    },
                }
            )
        except Exception as exc:
            raise HTTPException(status_code=500, detail=str(exc))

    async def stream_chat_response(platform: Any, message: str, model_name: str) -> Any:
        """Stream chat response in OpenAI-compatible format."""
        completion_id = f"chatcmpl-{str(uuid.uuid4())}"
        created_time = int(time.time())
        conversation_id = str(uuid.uuid4())

        # Send the initial chunk with the role
        initial_data = {
            "id": completion_id,
            "object": "chat.completion.chunk",
            "created": created_time,
            "model": model_name,
            "choices": [
                {"index": 0, "delta": {"role": "assistant"}, "finish_reason": None}
            ],
        }
        yield f"data: {json.dumps(initial_data)}\n\n"

        try:
            # Use the streaming-capable chat method
            response_generator = platform.chat(message)

            full_response = ""
            has_content = False

            # Iterate over the generator and stream chunks
            for chunk in response_generator:
                if chunk:
                    has_content = True
                    full_response += chunk
                    chunk_data = {
                        "id": completion_id,
                        "object": "chat.completion.chunk",
                        "created": created_time,
                        "model": model_name,
                        "choices": [
                            {
                                "index": 0,
                                "delta": {"content": chunk},
                                "finish_reason": None,
                            }
                        ],
                    }
                    yield f"data: {json.dumps(chunk_data)}\n\n"

            if not has_content:
                no_response_message = "No response from model."
                chunk_data = {
                    "id": completion_id,
                    "object": "chat.completion.chunk",
                    "created": created_time,
                    "model": model_name,
                    "choices": [
                        {
                            "index": 0,
                            "delta": {"content": no_response_message},
                            "finish_reason": None,
                        }
                    ],
                }
                yield f"data: {json.dumps(chunk_data)}\n\n"
                full_response = no_response_message

            # Send the final chunk with finish_reason
            final_data = {
                "id": completion_id,
                "object": "chat.completion.chunk",
                "created": created_time,
                "model": model_name,
                "choices": [{"index": 0, "delta": {}, "finish_reason": "stop"}],
            }
            yield f"data: {json.dumps(final_data)}\n\n"

            # Send the [DONE] marker
            yield "data: [DONE]\n\n"

            # Log the full conversation
            log_conversation(
                conversation_id,
                [{"role": "user", "content": message}],
                model_name,
                full_response,
            )

        except Exception as exc:
            error_msg = f"Error during streaming: {str(exc)}"
            PrettyOutput.print(error_msg, OutputType.ERROR)

            # Send error information in the stream
            error_chunk = {
                "id": completion_id,
                "object": "chat.completion.chunk",
                "created": created_time,
                "model": model_name,
                "choices": [
                    {
                        "index": 0,
                        "delta": {"content": error_msg},
                        "finish_reason": "stop",
                    }
                ],
            }
            yield f"data: {json.dumps(error_chunk)}\n\n"
            yield "data: [DONE]\n\n"

            # Log the error
            log_conversation(
                conversation_id,
                [{"role": "user", "content": message}],
                model_name,
                response=f"ERROR: {error_msg}",
            )

    # Run the server
    uvicorn.run(app, host=host, port=port)



# -*- coding: utf-8 -*-
import json
import threading
import time
from typing import Any, Callable, Dict, List, Optional
from urllib.parse import parse_qs, urlencode, urljoin

import requests

from jarvis.jarvis_mcp import McpClient
from jarvis.jarvis_utils.output import OutputType, PrettyOutput


class SSEMcpClient(McpClient):
    """远程MCP客户端实现

    参数:
        config: 配置字典，包含以下字段：
            - base_url: str - MCP服务器的基础URL
            - auth_token: str - 认证令牌（可选）
            - headers: Dict[str, str] - 额外的HTTP头（可选）
    """

    def __init__(self, config: Dict[str, Any]):
        self.config = config
        self.base_url = config.get("base_url", "")
        if not self.base_url:
            raise ValueError("No base_url specified in config")

        # 设置HTTP客户端
        self.session = requests.Session()
        self.session.headers.update(
            {
                "Content-Type": "application/json",
                "Accept": "application/json",
            }
        )

        # 添加认证令牌（如果提供）
        auth_token = config.get("auth_token")
        if auth_token:
            self.session.headers["Authorization"] = f"Bearer {auth_token}"

        # 添加额外的HTTP头
        extra_headers = config.get("headers", {})
        self.session.headers.update(extra_headers)

        # SSE相关属性
        self.sse_response: Optional[requests.Response] = None
        self.sse_thread: Optional[threading.Thread] = None
        self.messages_endpoint: Optional[str] = None
        self.session_id: Optional[str] = None
        self.pending_requests = {}  # 存储等待响应的请求 {id: Event}
        self.request_results = {}  # 存储请求结果 {id: result}
        self.notification_handlers = {}
        self.event_lock = threading.Lock()
        self.request_id_counter = 0

        # 初始化连接
        self._initialize()

    def _initialize(self) -> None:
        """初始化MCP连接"""
        try:
            # 启动SSE连接
            self._start_sse_connection()

            # 等待获取消息端点和会话ID
            start_time = time.time()
            while (
                not self.messages_endpoint or not self.session_id
            ) and time.time() - start_time < 5:
                time.sleep(0.1)

            if not self.messages_endpoint:
                self.messages_endpoint = "/messages"  # 默认端点
                PrettyOutput.print(
                    f"未获取到消息端点，使用默认值: {self.messages_endpoint}",
                    OutputType.WARNING,
                )

            if not self.session_id:
                PrettyOutput.print("未获取到会话ID", OutputType.WARNING)

            # 发送初始化请求
            response = self._send_request(
                "initialize",
                {
                    "processId": None,  # 远程客户端不需要进程ID
                    "clientInfo": {"name": "jarvis", "version": "1.0.0"},
                    "capabilities": {},
                    "protocolVersion": "2025-03-26",
                },
            )

            # 验证服务器响应
            if "result" not in response:
                raise RuntimeError(
                    f"初始化失败: {response.get('error', 'Unknown error')}"
                )

            # 发送initialized通知
            self._send_notification("notifications/initialized", {})

        except Exception as e:
            PrettyOutput.print(f"MCP初始化失败: {str(e)}", OutputType.ERROR)
            raise

    def _start_sse_connection(self) -> None:
        """建立SSE连接并启动处理线程"""
        try:
            # 设置SSE请求头
            sse_headers = dict(self.session.headers)
            sse_headers.update(
                {
                    "Accept": "text/event-stream",
                    "Cache-Control": "no-cache",
                }
            )

            # 建立SSE连接
            sse_url = urljoin(self.base_url, "sse")
            self.sse_response = self.session.get(
                sse_url, stream=True, headers=sse_headers, timeout=30
            )
            if self.sse_response:
                self.sse_response.raise_for_status()

            # 启动事件处理线程
            self.sse_thread = threading.Thread(
                target=self._process_sse_events, daemon=True
            )
            if self.sse_thread:
                self.sse_thread.start()

        except Exception as e:
            PrettyOutput.print(f"SSE连接失败: {str(e)}", OutputType.ERROR)
            raise

    def _process_sse_events(self) -> None:
        """处理SSE事件流"""
        if not self.sse_response:
            return

        buffer = ""
        for line in self.sse_response.iter_lines(decode_unicode=True):
            if line:
                if line.startswith("data:"):
                    data = line[5:].strip()
                    # 检查是否包含消息端点信息
                    if data.startswith("/"):
                        # 这是消息端点信息，例如 "/messages/?session_id=xyz"
                        try:
                            # 提取消息端点路径和会话ID
                            url_parts = data.split("?")
                            self.messages_endpoint = url_parts[0]

                            # 如果有查询参数，尝试提取session_id
                            if len(url_parts) > 1:
                                query_string = url_parts[1]
                                query_params = parse_qs(query_string)
                                if "session_id" in query_params:
                                    self.session_id = query_params["session_id"][0]
                        except Exception as e:
                            PrettyOutput.print(
                                f"解析消息端点或会话ID失败: {e}", OutputType.WARNING
                            )
                    else:
                        buffer += data
                elif line.startswith(":"):  # 忽略注释行
                    continue
                elif line.startswith("event:"):  # 事件类型
                    continue  # 我们不使用事件类型
                elif line.startswith("id:"):  # 事件ID
                    continue  # 我们不使用事件ID
                elif line.startswith("retry:"):  # 重连时间
                    continue  # 我们自己管理重连
            else:  # 空行表示事件结束
                if buffer:
                    try:
                        self._handle_sse_event(buffer)
                    except Exception as e:
                        PrettyOutput.print(f"处理SSE事件出错: {e}", OutputType.ERROR)
                    buffer = ""

        PrettyOutput.print("SSE连接已关闭", OutputType.WARNING)

    def _handle_sse_event(self, data: str) -> None:
        """处理单个SSE事件数据"""
        try:
            event_data = json.loads(data)

            # 检查是请求响应还是通知
            if "id" in event_data:
                # 这是一个请求的响应
                req_id = event_data["id"]
                with self.event_lock:
                    self.request_results[req_id] = event_data
                    if req_id in self.pending_requests:
                        # 通知等待线程响应已到达
                        self.pending_requests[req_id].set()
            elif "method" in event_data:
                # 这是一个通知
                method = event_data.get("method", "")
                params = event_data.get("params", {})

                # 调用已注册的处理器
                if method in self.notification_handlers:
                    for handler in self.notification_handlers[method]:
                        try:
                            handler(params)
                        except Exception as e:
                            PrettyOutput.print(
                                f"处理通知时出错 ({method}): {e}", OutputType.ERROR
                            )
        except json.JSONDecodeError:
            PrettyOutput.print(f"无法解析SSE事件: {data}", OutputType.WARNING)
        except Exception as e:
            PrettyOutput.print(f"处理SSE事件时出错: {e}", OutputType.ERROR)

    def register_notification_handler(self, method: str, handler: Callable) -> None:
        """注册通知处理器

        参数:
            method: 通知方法名
            handler: 处理通知的回调函数，接收params参数
        """
        with self.event_lock:
            if method not in self.notification_handlers:
                self.notification_handlers[method] = []
            self.notification_handlers[method].append(handler)

    def unregister_notification_handler(self, method: str, handler: Callable) -> None:
        """注销通知处理器

        参数:
            method: 通知方法名
            handler: 要注销的处理器函数
        """
        with self.event_lock:
            if method in self.notification_handlers:
                if handler in self.notification_handlers[method]:
                    self.notification_handlers[method].remove(handler)
                if not self.notification_handlers[method]:
                    del self.notification_handlers[method]

    def _get_next_request_id(self) -> str:
        """获取下一个请求ID"""
        with self.event_lock:
            self.request_id_counter += 1
            return str(self.request_id_counter)

    def _send_request(self, method: str, params: Dict[str, Any]) -> Dict[str, Any]:
        """发送请求到MCP服务器

        参数:
            method: 请求方法
            params: 请求参数

        返回:
            Dict[str, Any]: 响应结果
        """
        # 生成唯一请求ID
        req_id = self._get_next_request_id()

        # 创建事件标志，用于等待响应
        event = threading.Event()

        with self.event_lock:
            self.pending_requests[req_id] = event

        try:
            # 构建请求
            request = {
                "jsonrpc": "2.0",
                "method": method,
                "params": params,
                "id": req_id,
            }

            # 尝试不同的请求发送方式
            if self.session_id:
                # 方法1: 使用查询参数中的session_id
                query_params = {"session_id": self.session_id}
                messages_url = urljoin(self.base_url, self.messages_endpoint)

                # 尝试直接使用原始URL（不追加查询参数）
                try:
                    post_response = self.session.post(messages_url, json=request)
                    post_response.raise_for_status()
                except requests.HTTPError:
                    # 如果失败，尝试添加会话ID到查询参数
                    messages_url_with_session = (
                        f"{messages_url}?{urlencode(query_params)}"
                    )
                    post_response = self.session.post(
                        messages_url_with_session, json=request
                    )
                    post_response.raise_for_status()
            else:
                # 方法2: 不使用session_id
                if not self.messages_endpoint:
                    self.messages_endpoint = "/messages"

                messages_url = urljoin(self.base_url, self.messages_endpoint)

                # 尝试直接使用messages端点而不带任何查询参数
                try:
                    # 尝试1: 标准JSON-RPC格式
                    post_response = self.session.post(messages_url, json=request)
                    post_response.raise_for_status()
                except requests.HTTPError:
                    # 尝试2: JSON字符串作为请求参数
                    post_response = self.session.post(
                        messages_url, params={"request": json.dumps(request)}
                    )
                    post_response.raise_for_status()

            # 等待SSE通道返回响应（最多30秒）
            if not event.wait(timeout=30):
                raise TimeoutError(f"等待响应超时: {method}")

            # 获取响应结果
            with self.event_lock:
                result = self.request_results.pop(req_id, None)
                self.pending_requests.pop(req_id, None)

            if result is None:
                raise RuntimeError(f"未收到响应: {method}")

            return result

        except Exception as e:
            # 清理请求状态
            with self.event_lock:
                self.pending_requests.pop(req_id, None)
                self.request_results.pop(req_id, None)

            PrettyOutput.print(f"发送请求失败: {str(e)}", OutputType.ERROR)
            raise

    def _send_notification(self, method: str, params: Dict[str, Any]) -> None:
        """发送通知到MCP服务器（不需要响应）

        参数:
            method: 通知方法
            params: 通知参数
        """
        try:
            # 构建通知
            notification = {"jsonrpc": "2.0", "method": method, "params": params}

            # 尝试不同的请求发送方式，与_send_request保持一致
            if self.session_id:
                # 方法1: 使用查询参数中的session_id
                query_params = {"session_id": self.session_id}
                messages_url = urljoin(
                    self.base_url, self.messages_endpoint or "/messages"
                )

                # 尝试直接使用原始URL（不追加查询参数）
                try:
                    post_response = self.session.post(messages_url, json=notification)
                    post_response.raise_for_status()
                except requests.HTTPError:
                    # 如果失败，尝试添加会话ID到查询参数
                    messages_url_with_session = (
                        f"{messages_url}?{urlencode(query_params)}"
                    )
                    post_response = self.session.post(
                        messages_url_with_session, json=notification
                    )
                    post_response.raise_for_status()
            else:
                # 方法2: 不使用session_id
                if not self.messages_endpoint:
                    self.messages_endpoint = "/messages"

                messages_url = urljoin(self.base_url, self.messages_endpoint)

                # 尝试直接使用messages端点而不带任何查询参数
                try:
                    # 尝试1: 标准JSON-RPC格式
                    post_response = self.session.post(messages_url, json=notification)
                    post_response.raise_for_status()
                except requests.HTTPError:
                    # 尝试2: JSON字符串作为请求参数
                    post_response = self.session.post(
                        messages_url, params={"request": json.dumps(notification)}
                    )
                    post_response.raise_for_status()

        except Exception as e:
            PrettyOutput.print(f"发送通知失败: {str(e)}", OutputType.ERROR)
            raise

    def get_tool_list(self) -> List[Dict[str, Any]]:
        """获取工具列表

        返回:
            List[Dict[str, Any]]: 工具列表，每个工具包含以下字段：
                - name: str - 工具名称
                - description: str - 工具描述
                - parameters: Dict - 工具参数
        """
        try:
            response = self._send_request("tools/list", {})
            if "result" in response and "tools" in response["result"]:
                # 注意这里: 响应结构是 response['result']['tools']
                tools = response["result"]["tools"]
                # 将MCP协议字段转换为内部格式
                formatted_tools = []
                for tool in tools:
                    # 从inputSchema中提取参数定义
                    input_schema = tool.get("inputSchema", {})
                    parameters = {}
                    if "properties" in input_schema:
                        parameters = input_schema["properties"]

                    formatted_tools.append(
                        {
                            "name": tool.get("name", ""),
                            "description": tool.get("description", ""),
                            "parameters": parameters,
                        }
                    )
                return formatted_tools
            else:
                error_msg = "获取工具列表失败"
                if "error" in response:
                    error_msg += f": {response['error']}"
                elif "result" in response:
                    error_msg += f": 响应格式不正确 - {response['result']}"
                else:
                    error_msg += ": 未知错误"

                PrettyOutput.print(error_msg, OutputType.ERROR)
                return []
        except Exception as e:
            PrettyOutput.print(f"获取工具列表失败: {str(e)}", OutputType.ERROR)
            return []

    def execute(self, tool_name: str, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """执行工具

        参数:
            tool_name: 工具名称
            arguments: 参数字典，包含工具执行所需的参数

        返回:
            Dict[str, Any]: 执行结果，包含以下字段：
                - success: bool - 是否执行成功
                - stdout: str - 标准输出
                - stderr: str - 标准错误
        """
        try:
            response = self._send_request(
                "tools/call", {"name": tool_name, "arguments": arguments}
            )
            if "result" in response:
                result = response["result"]
                # 从content中提取输出信息
                stdout = ""
                stderr = ""
                for content in result.get("content", []):
                    if content.get("type") == "text":
                        stdout += content.get("text", "")
                    elif content.get("type") == "error":
                        stderr += content.get("text", "")

                return {"success": True, "stdout": stdout, "stderr": stderr}
            else:
                return {
                    "success": False,
                    "stdout": "",
                    "stderr": response.get("error", "Unknown error"),
                }
        except Exception as e:
            PrettyOutput.print(f"执行工具失败: {str(e)}", OutputType.ERROR)
            return {"success": False, "stdout": "", "stderr": str(e)}

    def get_resource_list(self) -> List[Dict[str, Any]]:
        """获取资源列表

        返回:
            List[Dict[str, Any]]: 资源列表，每个资源包含以下字段：
                - uri: str - 资源的唯一标识符
                - name: str - 资源的名称
                - description: str - 资源的描述（可选）
                - mimeType: str - 资源的MIME类型（可选）
        """
        try:
            response = self._send_request("resources/list", {})
            if "result" in response and "resources" in response["result"]:
                return response["result"]["resources"]
            else:
                error_msg = "获取资源列表失败"
                if "error" in response:
                    error_msg += f": {response['error']}"
                else:
                    error_msg += ": 未知错误"
                PrettyOutput.print(error_msg, OutputType.ERROR)
                return []
        except Exception as e:
            PrettyOutput.print(f"获取资源列表失败: {str(e)}", OutputType.ERROR)
            return []

    def get_resource(self, uri: str) -> Dict[str, Any]:
        """获取指定资源的内容

        参数:
            uri: str - 资源的URI标识符

        返回:
            Dict[str, Any]: 执行结果，包含以下字段：
                - success: bool - 是否执行成功
                - stdout: str - 资源内容（文本或base64编码的二进制内容）
                - stderr: str - 错误信息
        """
        try:
            response = self._send_request("resources/read", {"uri": uri})
            if "result" in response and "contents" in response["result"]:
                contents = response["result"]["contents"]
                if contents:
                    content = contents[0]  # 获取第一个资源内容
                    # 根据资源类型返回内容
                    if "text" in content:
                        return {
                            "success": True,
                            "stdout": content["text"],
                            "stderr": "",
                        }
                    elif "blob" in content:
                        return {
                            "success": True,
                            "stdout": content["blob"],
                            "stderr": "",
                        }
                return {"success": False, "stdout": "", "stderr": "资源内容为空"}
            else:
                error_msg = "获取资源内容失败"
                if "error" in response:
                    error_msg += f": {response['error']}"
                else:
                    error_msg += ": 未知错误"
                PrettyOutput.print(error_msg, OutputType.ERROR)
                return {"success": False, "stdout": "", "stderr": error_msg}
        except Exception as e:
            error_msg = f"获取资源内容失败: {str(e)}"
            PrettyOutput.print(error_msg, OutputType.ERROR)
            return {"success": False, "stdout": "", "stderr": error_msg}

    def __del__(self):
        """清理资源"""
        # 清理请求状态
        with self.event_lock:
            for event in self.pending_requests.values():
                event.set()  # 释放所有等待的请求
            self.pending_requests.clear()
            self.request_results.clear()

        # 关闭SSE响应
        if self.sse_response:
            try:
                self.sse_response.close()
            except:
                pass

        # 关闭HTTP会话
        if self.session:
            self.session.close()
